/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#ifndef BASE_TRAIT_LODS_H
#define BASE_TRAIT_LODS_H

#ifdef  DEFINE_DATA

/******************************************************************************/
/*                                                                            */
/* Define a structure for keeping the null lod score contributions in.        */
/*                                                                            */
/******************************************************************************/

struct base_trait_lods_elem {           /* Base_trait_lods_elem:              */
    double contrib;                     /* Null lod score contribution.       */
                            };
typedef struct base_trait_lods_elem BTLE_ELEM;
typedef BTLE_ELEM * BTLE_PTR;

/******************************************************************************/
/*                                                                            */
/* Storage for all the base_trait_lods_elems.  This is a two dimensional      */
/* matrix with number of components rows and number of realizations columns.  */
/* Each entry in the matrix is a base_trait_lods_elem (BTL_ELEM).             */
/*                                                                            */
/******************************************************************************/

BTLE_ELEM **btle_storage;           /* Storage for all base_trait_lods_elems. */

/* Three vectors are needed to save cumulative base log-likelihoods, lod      */
/* scores and standard errors of the lod scores for each component.           */

double *comp_log_like;
double *comp_lod_scor;
double *comp_sd;

/******************************************************************************/
/*                                                                            */
/* Pointers to two additional vectors to save the results returned by the     */
/* pp_lods subroutine in.                                                     */
/*                                                                            */
/******************************************************************************/

double *pp_log_like;
double *pp_lod_scor;

/******************************************************************************/
/*                                                                            */
/* The following two variables are for keeping the subscripts into the rows   */
/* of the btle_storage matrix where the 5th percent and the 95th percent      */
/* quantiles are.                                                             */
/*                                                                            */
/******************************************************************************/

int   quant_05;                         /*  5th percent quantile subscript.   */
int   quant_95;                         /* 95th percent quantile subscript.   */

#endif  /*DEFINE_DATA*/

/* Subroutines currently in the base_trait_rtnes.c file:                      */

double **bt_setupInherit (int);
double do_mclik_cmp (CTP_PTR, int, int, double, BTLE_PTR);
void get_mc_realizations (CTP_PTR, int);
void mc_lods_results (int, double, double *, double *);
void mc_lods (CTP_PTR, int, double *, double *);
void pp_lods (int, double *, double *);
void alloc_btl_stor (void);
void free_btl_stor (void);
void set_quant_indices (void);
void print_btl_row (BTLE_PTR, double, int);
void print_btl_comp_result (int);
int  comparBtls (const void *, const void *);

struct _cleanup clean_base_trait[] = {
{"o   ", free_fnams},
{"o   ", freeChkPed},
{"o   ", freeKey},
{"    ", free_auto},
{"o   ", free_marek},
{"o   ", free_locnams},
{"o   ", free_ped},
{"o   ", free_nghds},
{"o   ", free_cmplists},
{"o   ", free_xxgg},
{"o   ", free_switches},
{"o   ", free_allefrq},
{"o   ", locus_peel_free},
{"o   ", free_allocate_loci},
{"    ", free_msmpl},
{"    ", free_sampling_cmp},
{"o   ", free_trait_pars},
{"    ", free_traits},
{"o   ", free_gen_pen},
{"    ", free_exact},
{"    ", free_dub},
{""  , NULL}};

#endif  /*BASE_TRAIT_LODS_H*/
