/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*      errsprogs.h:  error messages produced by MORGAN               */
/*                    main programs and subroutine libraries          */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef ERRSPROGS_H
#define ERRSPROGS_H

/**
 * @addtogroup Libraries
 * @{
 */

/* error codes which result in the error messages defined in           */
/* defined in Errrs[][] below                                          */

enum progEnums
    { ERR_nul,
      ERR_err,
      ERR_qnt,
      ERR_siz,
      ERR_nex,
      ERR_urd,
      ERC_dp1,
      ERC_dup,
      ERC_1p1,
      ERC_1pr,
      ERC_0p1,
      ERC_0pr,
      ERC_uas,
      ERC_1bp,
      ERC_bpr,
      ERC_ch1,
      ERC_chr,
      ERR_inh,
      ERR_mtx,
      ERR_mxg,
      ERR_mxe,
      ERR_nmi,
      ERR_nsm,
      ERQ_noi,
      ERR_noo,
      ERC_frm,
      ERC_fbi,
      ERC_fbr,
      ERC_fri,
      ERC_frr,
      ERC_nof,
      ERC_fin,
      ERC_fsx,
      ERC_wpf,
      ERC_cco,
      ERC_csa,
      ERC_coi,
      ERA_mdp,
      ERA_nmd,
      ERA_ncd,
      ERA_pnc,
      ERA_mcd,
      ERA_spg,
      ERR_mpg,
      ERR_nwi,
      ERA_mpg,
      ERA_npg,
      ERA_stm,
      ERA_inp,
      ERA_nff,
      ERA_naf,
      ERA_nip,
      ERA_nic,
      ERA_mca,
      ERA_nap,
      ERA_naa,
      ERA_mid,
      ERA_wgt,
      ERA_mrk,
      ERA_ngd,
      ERC_lcm,
      ERD_nof,
      ERD_igd,
      ERD_ixf,
      ERD_guz,
      ERL_bt1,
      ERL_bt2,
      ERS_zpp,
      ERS_zpl,
      ERN_cmp,
      ERN_sx1,
      ERN_sex,
      ERN_1pt,
      ERN_ngh,
      ERN_xpa,
      ERN_xma,
      ERN_nop,
      ERN_nam,
      ERN_mng,
      ERN_sxm,
      ERN_sxf,
      ERN_cfd,
      ERN_ccp,
      ERQ_flm,
      ERQ_hff,
      ERQ_rge,
      ERQ_rse,
      ERQ_nfe,
      ERQ_rfe,
      ERP_con,
      ERP_nzd,
      ERP_swc,
      ERP_npv,
      ERP_nnp,
      ERP_nav,
      ERP_ueo,
      ERK_klc,
      ERK_kmc,
      ERS_fex,
      ERS_fxr,
      ERS_fop,
      ERS_fof,
      ERS_pfe,
      ERS_mal,
      ERS_mxw,
      ERS_flz,
      ERR_qit } ;

#ifdef DEFINE_DATA
char *Errrs[] = {
 "***" ,                                                             /*ERR_nul*/
 " %s" ,                                                             /*ERR_err*/
 " %s():  Calling program error:  \"quant\" not malloced",           /*ERR_qnt*/
 " %s():  Pedigree size must be greater than 2" ,                    /*ERR_siz*/
 " %d individuals expected" ,                                        /*ERR_nex*/
 " %s():  Unread record(s) remain in pedigree file" ,                /*ERR_urd*/
 " %s():  1 duplicate name, %s, was found" ,                         /*ERC_dp1*/
 " %s():  %d duplicate names were found\n"
 "              %s was the first duplicate name found",              /*ERC_dup*/
 " %s():  Individual %s has only one parent named" ,                 /*ERC_1p1*/
 " %s():  %d individuals have only one parent named\n"
 "                  %s was the first such individual" ,              /*ERC_1pr*/
 " %s():  Named parent(s) of individual %s are not in pedigree" ,    /*ERC_0p1*/
 " %s():  Named parent(s) of %d individuals are not in pedigree\n"
 "                %s was the first such individual found" ,          /*ERC_0pr*/
 " Unsuccessful assignment of gender:" ,                             /*ERC_uas*/
 " %s():  %d individual, %s, has parent(s) of wrong sex" ,           /*ERC_1bp*/
 " %s():  %d individuals have parent(s) of wrong sex\n"
 "                 %s was the first such individual" ,               /*ERC_bpr*/
 " %s():  Unable to assign chronological order to individual %s",    /*ERC_ch1*/
 " %s():  Unable to assign chronological order to %d individuals\n"
 "                %s was the first unassigned name" ,                /*ERC_chr*/
 " %s():  Failed for individual %s" ,                                /*ERR_inh*/
 " %s():  Matrix not positive definite" ,                            /*ERR_mtx*/
 " %s():  Genetic covariance matrix not positive definite" ,         /*ERR_mxg*/
 " %s():  Environmental covariance matrix not positive definite",    /*ERR_mxe*/
 " Neither output of meiosis indicators nor founder genome labels\n"
 "         has been chosen, so gl_auto has nothing to output" ,      /*ERR_nmi*/
 " No MCMC-scoreable meioses were found in any pedigree component",  /*ERR_nsm*/
 " No observed individuals were found in pedigree",                  /*ERQ_noi*/
 " A pedigree component with %d member(s) has no observed individuals\n"
 " %s is the first member of this component",                        /*ERR_noo*/
 " Not enough data in pedigree or individual record #%d" ,           /*ERC_frm*/
 " \"%s\" is not valid integer in pedigree record #%d" ,             /*ERC_fbi*/
 " \"%s\" is not valid real number in pedigree record #%d" ,         /*ERC_fbr*/
 " Range error for integer \"%s\" in pedigree record #%d" ,          /*ERC_fri*/
 " Range error for real number \"%s\" in pedigree record #%d" ,      /*ERC_frr*/
 " Unable to assign output pedigree file\n"
 " Please provide output file name in parameter statement or command line",
                                                                     /*ERC_nof*/
 " Bad name \"%s\" in pedigree record #%d" ,                         /*ERC_fin*/
 " Bad gender \"%s\" in pedigree record #%d" ,                       /*ERC_fsx*/
 " A new pedigree file will be written",                             /*ERC_wpf*/
 " The input pedigree was not by component and/or in chronological order",
                                                                     /*ERC_cco*/
 " There are changes to the pedigree in assignment of gender",       /*ERC_csa*/
 " There are changes to the pedigree in the observed indicator",     /*ERC_coi*/
 " Marker data given for individual %s, who is not in the pedigree", /*ERA_mdp*/
 " No marker data for component %d" ,                                /*ERA_nmd*/
 " No proband gametes are specified for component %d" ,              /*ERA_ncd*/
 " Proband %s is not member of component %d:"
                        "  it was found in component %d",            /*ERA_pnc*/
 " Proband gametes given for %d components;\n"
 "    %d component(s) read from pedigree file " ,                    /*ERA_mcd*/
 " No \"set proband gametes ...\" statement has been found" ,        /*ERA_spg*/
 " Proband %s was not found among the names in the marker data file",
                                                                     /*ERR_mpg*/
 " In scoreset %d \"%s\" is not a whole individual" ,                /*ERR_nwi*/
 " Did not find any named probands in the pedigree file" ,           /*ERA_mpg*/
 " Did not find any named probands in the marker data file" ,        /*ERA_npg*/
 " The 2009 state transition matrix is not implemented for %d states",
                                                                     /*ERA_stm*/
 " Proband %s is not a member of the pedigree" ,                     /*ERA_inp*/
 " There are no affected individuals in pedigree file for component %d" ,
                                                                     /*ERA_nff*/
 " No affected individuals are specified for component %d" ,         /*ERA_naf*/
 " Affected individual \"%s\" is not a member of the pedigree" ,     /*ERA_nip*/
 " Affected individual \"%s\" is not member of component %d:\n"
 "    it was found in component %d",                                 /*ERA_nic*/
 " Affected individuals named for %d components;\n"
 "    %d component(s) read from pedigree file " ,                    /*ERA_mca*/
 " No affected individuals (with genotype code (3 or 4)) were found\n"
 "    in the pedigree file" ,                                        /*ERA_nap*/
 " There are no affected individuals" ,                              /*ERA_naa*/
 " Mendelian inconsistency detected in component %d at marker %d",   /*ERA_mid*/
 " Zero probability in peeling operation for locus %d" ,             /*ERA_wgt*/
 " There is a problem with the no. of trait and marker positions:\n"
 "    NLocPos = %d,  ntp = %d" ,                                     /*ERA_mrk*/
 " A \"display\" was requested, but the program was not linked\n"
 "    with the graphics libraries.  Use \"make morgan.gcc.dsp\"." ,  /*ERA_ngd*/
 " Individual %s's liability class = %d; this is greater than the number\n"
 "    of liability classes defined = %d     ",                       /*ERC_lcm*/
 " Please provide an output scores file name in a parameter statement or\n"
 " on the command line",                                             /*ERD_nof*/
 " Please provide an input gamete data file name in a parameter statement\n"
 " or on the command line",                                          /*ERD_igd*/
 " Please provide an input extra file name in a parameter statement or\n"
 " on the command line",                                             /*ERD_ixf*/
 " An error occurred in the gunzipit subroutine",                    /*ERD_guz*/
 " Number of batches (%d) does not divide scored iterations" ,       /*ERL_bt1*/
 " Batch size is not > 0; there must be at least %d scored iterations",
                                                                     /*ERL_bt2*/
 " Zero probability in peeling operation" ,                          /*ERS_zpp*/
 " Zero probability at locus %d" ,                                   /*ERS_zpl*/
 " Problem:  %s() called even though all individuals are already"
                       " in components" ,                            /*ERN_cmp*/
 " %d incorrectly sexed parent" ,                                    /*ERN_sx1*/
 " %d incorrectly sexed parents" ,                                   /*ERN_sex*/
 " Individual %s has only one parent" ,                              /*ERN_1pt*/
 " No neighborhood for individual %s.  Oops!" ,                      /*ERN_ngh*/
 " %s():  Impossible father for %d" ,                                /*ERN_xpa*/
 " %s():  Impossible mother for %d" ,                                /*ERN_xma*/
 " %s():  Warning! Have no individual in the pedigree"
                     " corresponding to row_index %d" ,              /*ERN_nop*/
 " Name %d out of range " ,                                          /*ERN_nam*/
 " %s():  Tried to make nghd twice for %s" ,                         /*ERN_mng*/
 " %s():  Sex change operation for mother %s" ,                      /*ERN_sxm*/
 " %s():  Sex change operation for father %s" ,                      /*ERN_sxf*/
 " Pedchk found %d founders\n"
 "    Nghds found %d founders" ,                                     /*ERN_cfd*/
 " Pedchk found %d components\n"
 "    Nghds found %d components" ,                                   /*ERN_ccp*/
 " Civil does not currently permit more than 15 founders" ,          /*ERQ_flm*/
 " %s():  CAUTION;  program not guaranteed for half-founder %d" ,    /*ERQ_hff*/
 " Error return from rg_() is %d" ,                                  /*ERQ_rge*/
 " Error return from rs_() is %d" ,                                  /*ERQ_rse*/
 " Fixed effects categories must be positive or 0 (unobserved);\n"
 "    negative value encountered for fixed effect %d, individual %s",/*ERQ_nfe*/
 " Maximum fixed-effects category is %d;  out-of-range value\n"
 "    encountered for fixed effect %d, individual %s" ,              /*ERQ_rfe*/
 " DATA ERROR:  non-simply connected pedigree" ,                     /*ERP_con*/
 " ERROR:  %d individuals have non-zero degree" ,                    /*ERP_nzd*/
 " Cannot switch dominance models" ,                                 /*ERP_swc*/
 " %s():  Warning;  negative pivot found at %d" ,                    /*ERP_npv*/
 " %s():  # negative pivots = %d" ,                                  /*ERP_nnp*/
 " %s():  WARNING: why polygenic peel with no additive variance ??\n"
 "            use mjrpeel to peel major gene models\n"
 "            use envonly to compute likelihoods for non-genetic model" ,
                                                                     /*ERP_nav*/
 " Using envonly as you maybe meant to gives %f" ,                   /*ERP_ueo*/
 " Probands given for %d component(s), which is less than the number read" ,
                                                                     /*ERK_klc*/
 " Probands given for %d component(s), which is greater than the number read" ,
                                                                     /*ERK_kmc*/
 " %c%s \"%s\" already exists\n"
 " Writing to it would destroy its contents" ,                       /*ERS_fex*/
 " %c%s \"%s\" already exists; it will be overwritten" ,             /*ERS_fxr*/
 " Can't open \"%s\", mode \"%s\",  %s" ,                            /*ERS_fop*/
 " Can't open %s \"%s\"  %s" ,                                       /*ERS_fof*/
 " Expected pedigree file to contain 0, 1 or more parameter statements,\n"
 "     terminated by a line starting with \"%s\",\n"
 "     followed by pedigree data" ,                                  /*ERS_pfe*/
 " Malloc failed:  %s" ,                                             /*ERS_mal*/
 " Maximum number of warning messages permitted has been exceeded" , /*ERS_mxw*/
 " Field width for function must be at least 3 > no. decimal places",/*ERS_flz*/
 " Program quitting\n\n" } ;                                         /*ERR_qit*/
#else

  extern char *Errrs[];

#endif  /*DEFINE_DATA*/

/** @} */

#endif     /*ERRSPROGS_H*/
