/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#ifndef GFMCMC_DRAW_H
#define GFMCMC_DRAW_H
/**
 * @file   GFMCMC_DrawingUtilities.h
 * @author Eric C. Anderson
 * @date   Fri, 30 Aug 2002 17:43:31 -0700
 *
 * @brief  Utitilies for drawing. 
 *
 *
 * $Id: gfmcmc_draw.h,v 1.3 2005/08/14 00:19:36 thompson Exp $
 **/
/**
 * @addtogroup GFMCMC
 * @{
 */

#include "mcTypesEtc.h"

#define GFMDU_NO_STIPPLE 65535


/*  here we define a data structure that is useful for watching traces of variables */
/*  but doesn't require a huge amount of data storage because the window slides */
/*  forward with iteration number */
typedef struct
{
   int Niter;  /**< the number of iterations to store */
   int Lo;     /**< the value of the lowest iteration number stored */
   int Hi;     /**< the value of the highest iteration number stored */
   double Span;

   int Start;  /**< the index of the starting value  */
   int Interval;  /**< the interval at which one wants to record information */

   double *Trace;  /**< the array of values.  This gets dynamically allocated
                     to */

   double OverallMax;  /**< the overall maximum observed since last refreshing
                         of averages */

   double OverallMin;  /**< the overall minimum observed since last refreshing
                         of averages */

} sliding_trace_struct;




/*  prototypes */

void 
gfmduDrawOneHistAsCurveWith2Bars( dval *D,
                                  GLint factor,
                                  GLushort pattern,
                                  int AsDensity );

/*  sliding trace stuff */
sliding_trace_struct *gfduAllocSlidingTrace(int Length, int Interval);
void gfduIncrementSlidingTrace(sliding_trace_struct *S, double value);
void gfduDrawSlidingTrace(sliding_trace_struct *S);

void gfduInitSlidingTraceToZero(sliding_trace_struct *temp);

/**
 *
 *
 * @param x1
 * @param y1
 * @param x2
 * @param y2
 *
 */
void gfduWireRectf(GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2);

/** @} */

/* vim: sw=3
*/
#endif /* GFMCMC_DRAW_H */
