/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved.     */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.    */
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml          */

/******************************************************************************/
/*                                                                            */
/*       parsedefs.h:  header file for the Pars/ library                      */
/*                 -  of file scope                                           */
/*                 -  included by Pars/ functions                             */
/*                 -  contains definitions of parameter statements            */
/*                    and command line paramters for main programs            */
/*                 -  list of functions called by parameter statements        */
/*                    is included from parsedefs_fncs.h                       */
/*                 -  definitions of statements are included from             */
/*                    parsedefs_d.h                                           */
/*                 -  texts of statments are included from                    */
/*                    parsedefs_f.h                                           */
/*                 -  definitions of obsolete statements are included from    */
/*                    parsedefs_obsl.h                                        */
/*                                                                            */
/******************************************************************************/

#ifndef  PARSEDEFS_H
#define  PARSEDEFS_H

/**
 * @addtogroup parse
 * @{
 */

#define KEYWD 0                         /*Token is a keyword                  */
#define PDAT 1                          /*Token is data                       */
#define MAXNKEY 10                      /*The max # of keywords in a statement*/
#define MAXNTOKEN 200                   /*The max # of tokens in a statement  */
                                        /*(initial no. only, may be realloc)  */
#define NOMTOKLEN  8                    /*Nominal # of characters per token   */
                                        /*in an parameter statement           */

#define COM_LIN_FORM  "<prognam> <parfile>"

/*  line_in[]   - the next line of input data                                 */
/*  token[][]   - the tokens found in the input statemnt                      */
/*  token[]     - a list of pointers to the start of each token               */
/*  tok_typ[]   - corresponding token type                                    */
/*                (alphabetic, numeric or string)                             */
/*  aggre_key[] - the aggregate key                                           */
/*                (concatenated first 4 characters of each keyword)           */
/*  key_ix      - the statement type, as determined by the aggregate key      */

static char     line_in[MAXLINLEN+2];
static char  ** token;
static int    * tok_typ;
static char     aggre_key[MAXNKEY*4+1];
static int      key_ix;

/*  Function prototypes for functions which correspond to the various         */
/*  parameter statement types.                                                */

/*  for later, note that a typedef can be used;
 *  try typedef static int PARSTAT_FUNC (void)
 *  PARSTAT_FUNC allow_cmpsiz, allow_obsiz, ...
 */

/* List of functions is included from file parsedefs_fncs.h                   */

static int
#include "parsedefs_fncs.h"         /* List of functions for parameter
                                     * statements here.                       */
    not_impl       (void);          /* This function is for statements not
                                     * implemented.                           */

/*  prototype for input statement                                             */
/*  proto.key[]:  string containing the first four letters of each            */
/*                keyword in the statement.                                   */
/*  proto.par:    a code for each data value or list of values in the         */
/*                statement.                                                  */
/*                "i" - an integer value             example:   3000          */
/*                "k" - a list of integer values                2 4 5         */
/*                "h" - a hexadecimal value                     0x73331ad7    */
/*                "r" - a real value                            22.33         */
/*                "x" - a list of real values                   .3 .2 .9      */
/*                "s" - a string                                "pedFil3"     */
/*                "n" - a list of names                         A313 8333     */
/*                "g" - alternating names & integers                          */
/*                      (e.g. proband gametes)                  C16 0 D26 1   */
/*                "a" - sets of name plus list of integers,                   */
/*                      with the no. of sets specified by "i"                 */
/*                      (e.g. marker data)                      A1 3 5 3 3    */
/*                                                              B3 3 2 3 3    */

/*  proto.func:   the name of the function which acts upon the data           */
/*                from the statement                                          */
/*  proto.count:  the number of times this statement type occurs in the       */
/*                input file                                                  */

static char *nam_cod = "nga";       /* Codes for data lists containing names. */

/* pcods below are:                                                           */

/*       u - univar,      g - unibig,      b - bivar,        m - multivar,    */
/*       p - pedchk,      k - kin,                                            */
/*       d - genedrop,    i - ibddrop,     r - markerdrop,                    */
/*       q - lm_map,      v - lm_pval,     t - lm_ibdtests,  a - lm_auto,     */
/*       y - lm_bayes,    e - lm_linkage,  w - lm_twoqtl,    n - gl_lods,     */
/*       l - lm_lods,     s - lm_schnell,                                     */
/*       c - ibd_create,  h - ibd_haplo,                     x - phoenix,     */

/* Contents of table are included from file parsedefs_d.h                     */

static struct _proto {
       char * key;
       char * par;
       char * pcods;
       int (* func) (void);
       int    count;
               } *ppp;

static struct _proto  proto[] = {
#include "parsedefs_d.h"            /* Main body of table included here.      */
{""  ,""  ,""    ,not_impl    ,0}}; /* The last line is the sentinel.         */

/* Descriptions of input statements, SAME ORDER and SAME NUMBER               */
/* as the statement prototypes above.                                         */

/* Contents of table are included from file parsedefs_f.h                     */

static char * input_statement[] = {
#include "parsedefs_f.h"            /* main body of table included here       */
 ""                              }; /* the last line is the sentinel          */

/* Contents of table are included from file parsedefs_obsl.h                  */

static char * old_stat [] = {
#include "parsedefs_obsl.h"         /* main body of table included here       */
""                               }; /* the last line is the sentinel          */

/* Descriptions of input statements, SAME ORDER and SAME NUMBER               */
/* as the statement prototypes above.                                         */

/* Contents of table are included from file parsedefs_f.h                     */

/* These words may begin a new input statement, thus delimiting the           */
/* previous statement.                                                        */

static char * verb[] = {
             "allo",                /* allow        */
             "assi",                /* assign       */
             "assu",                /* assume       */
             "chec",                /* check        */
             "comp",                /* compute      */
             "disp",                /* display      */
             "fit",                 /* fit          */
             "igno",                /* ignore       */
             "inpu",                /* input        */
             "limi",                /* limit        */
             "map",                 /* map          */
             "outp",                /* output       */
             "samp",                /* sample       */
             "sele",                /* select       */
             "set",                 /* set          */
             "simu",                /* simulate     */
             "star",                /* start        */
             "use",                 /* use          */
             "****",                /* end-of-file  */
             ""     };

/** @} */

#endif  /*PARSEDEFS_H*/

