/* Program to parse the BEAGLE DAG in order to easily compute local
 *   haplotype frequencies.
 * Three components -- the edges --as in raw dag -- but with global index
 *          -- ordered by level and by parent within level
 *                  -- the nodes -- referencng to edges
 *                  -- the markers -- referencing start, and # edges,
 *                                 -- and start and number parent node index
 */

#define PNAM "ibd_create"
#define BUFLEN 256

#include "parsie.h"
#include "ibd_create.h"

typedef struct _Hap
{                                       /* partial haplotype at a cutset node */
   int   length;                        /* number of accumulaated markers */
   int   label;                         /* binary specification of alleles */
   double prob;                         /* node-specific accumulated probability   */
} Hap;

typedef struct _BGcutnode
{                                       /* cutset node in peeling */
   int   index;                         /* index in DAG node list */
   struct _Haplist *haps;
} BGcutnode;

typedef struct _Haplist
{
   struct _Hap *current;
   struct _Haplist *next;
} Haplist;

typedef struct _BGedge
{                                       /* basic DAG edge */
   int   index;
   int   parent;                        /*  parent node index  */
   int   child;                         /*  child node index   */
   int   level;                         /*  marker index  */
   char *marker;                        /*  marker name   */
   int   allele;                        /* allelic type of edge -- converted to int */
   int   count;                         /* count of haplotypes on this edge  */
   double prob;                         /* conditional probability of traversing the edge */
} BGedge;

typedef struct _BGnode
{                                       /* basic DAG node */
   int   index;
   int   parent;                        /*  parent marker index  */
   int   child;                         /*  child marker index -- needed ??  */
   int   startedge;                     /*  index of first out edge from this node */
   int   count;                         /* count of haps at this node */
   double prob;                         /* proportion of haplotypes in this cluster  */
} BGnode;

typedef struct _BGmarker
{                                       /* marker index into DAG */
   int   idxnode;                       /*  starting node index */
   int   idxedge;                       /*  starting edge index */
   int   countnode;                     /*  number of node indices at parent */
   int   countedge;                     /*  number of edge indices for this marker */
   double freq;                         /* frequency of allele 1 */
} BGmarker;

int   numedges, nummrks, numnodes;
Hap  *haplo;                            /* working haplotype, currently being peeled  */

BGedge *edges;
BGnode *nodes;
BGmarker *mrks;

BGcutnode *in_cutset;                   /*  vector of input cutset nodes  */
BGcutnode *out_cutset;                  /*  vector of output cutset nodes  */

int peel_marker (int l, BGcutnode * in_set, BGcutnode * out_set)
/*  Peeling marker l, from cluster level l to l+1   */
/* peeling operation -- will require adding haps to the nodes of level l+1 */
{
   int   k, j;                          /* k counts nodes of the cutset, j in original node index */
   int   i;                             /*  i counts across edges */
   int   iex, ief;                      /* starting and (ending+1) edges out of the node at l     */
#if 0
   int   onx, onn;                      /* Starting node and number of nodes  */
   /*                                      at level (l + 1).                  */
#endif

   double ptrans;                       /* conditional probability for current edge  */
   int   allele;                        /* allele on the current edge  */
   int   chout;                         /* output index of child node -- indexed back to cutnode */
   Haplist *hi;                         /* haplotypes at an input cutset node                    */
   Haplist *hh;                         /* haplotypes at an output cutset node                   */
   Haplist *hc;                         /* pointer storing current context in building Haplist hh */

#if 0
   onx = mrks[l + 1].idxnode;
   onn = mrks[l + 1].countnode;
#endif

/* for each node of in_cutset  */
   for (k = 0; k < mrks[l].countnode; ++k)
   {
      j = in_set[k].index;
      printf ("starting input cutset node %d\n", j);
      fflush (stdout);
/* index into nodes  to find the outedges from that node */
      iex = nodes[j].startedge;
      ief = nodes[j + 1].startedge;
      printf ("node has outedges starting at %d ending at %d\n", iex, ief - 1);
      fflush (stdout);

/*  for each out-edge from that node
    get the transition prob and the allele of the edge,
    and the level l+1 child node   (out_cutset)    */
      for (i = iex; i < ief; ++i)
      {
         ptrans = edges[i].prob;
         allele = edges[i].allele;
         chout = edges[i].child;
         printf ("edge %d has prob %8.4f allele %d, and goes to node %d  ",
                 i, ptrans, allele, chout);
         fflush (stdout);
         chout = chout - mrks[l + 1].idxnode;
         printf (" or %d in output cutset\n", chout);
         fflush (stdout);
         /* chout should now be the within-cutset index in output cutset */

         hi = in_set[k].haps;
         while (hi)
         {
/*  For each haplotype at this input cutset node,
        accumulate the length, label and prob of the haplo.
 */
            printf (" in-label  %d", hi->current->label);
            haplo->length = hi->current->length + 1;
            haplo->label = 2 * (hi->current->label) + (allele - 1);
            haplo->prob = (hi->current->prob) * ptrans;
            printf (" new length %d, label %d, prob %8.4e\n",
                    haplo->length, haplo->label, haplo->prob);
            fflush (stdout);
/*   go through the haps of the child node at level l+1 in out_cutset
 *     if      label is found, increment prob
 *     else    set up new Hap on end of the list
 */
            hh = out_set[chout].haps;
            while (hh)
            {
               if ((hh->current->length == -1) ||
                   (hh->current->label) == (haplo->label))
               {
                  hh->current->prob += haplo->prob;
                  hh->current->label = haplo->label;
                  hh->current->length = haplo->length;
                  printf (" found or init hap label %d, total prob %8.4f\n",
                          haplo->label, hh->current->prob);
                  haplo->length = -1;
                  break;
               }
               hh = hh->next;
            }
            if ((haplo->length) + 1)
/*  add the new hap to outset[chout].haps;
 */
            {
               hh = out_set[chout].haps;
               while (hh)
               {
                  hc = hh;
                  hh = hh->next;
               }
               hh = hc;
               printf (" hap not found: set new label %d prob %8.4f\n",
                       haplo->label, haplo->prob);
               fflush (stdout);
               hh->next = (Haplist *) malloc (sizeof (Haplist));
               hh = hh->next;
               hh->current = (Hap *) malloc (sizeof (Hap));
               hh->current->label = haplo->label;
               hh->current->prob = haplo->prob;
               hh->current->length = haplo->length;
               hh->next = NULL;
            }
            printf ("\n");
            fflush (stdout);
            hi = hi->next;
         }
      }
   }

   return 0;
}

int pool_hapsets (int nn, BGcutnode * in_set, BGcutnode * out_set)
/*  Pooling the hapsets at the final marker of the range  */
{
   int   k;                             /* k counts nodes of the input cutset */

   Haplist *hi;                         /* haplotypes at an input cutset node                    */
   Haplist *hh;                         /* haplotypes at an output cutset node                   */
   Haplist *hc;                         /* pointer storing current context in building Haplist hh */

/* for each node of in_cutset  */
   for (k = 0; k < nn; ++k)
   {
      hi = in_set[k].haps;
      while (hi)
      {
/*  For each haplotype at this input cutset node,
        recors the length, label and prob of the haplo.
 */
         printf ("   %d", hi->current->label);
         haplo->length = hi->current->length;
         haplo->label = hi->current->label;
         haplo->prob = hi->current->prob;
         printf ("  %8.4e\n", haplo->prob);
         fflush (stdout);
/*   go through the haps of the child node at level l+1 in out_cutset
 *     if      label is found, increment prob
 *     else    set up new Hap on end of the list
 */
         hh = out_set->haps;
         while (hh)
         {
            if ((hh->current->length == -1) ||
                (hh->current->label) == (haplo->label))
            {
               hh->current->prob += haplo->prob;
               hh->current->label = haplo->label;
               hh->current->length = haplo->length;
               haplo->length = -1;
               break;
            }
            hh = hh->next;
         }
         if ((haplo->length) + 1)
/*  add the new hap to outset.haps;
 */
         {
            hh = out_set->haps;
            while (hh)
            {
               hc = hh;
               hh = hh->next;
            }
            hh = hc;
            hh->next = (Haplist *) malloc (sizeof (Haplist));
            hh = hh->next;
            hh->current = (Hap *) malloc (sizeof (Hap));
            hh->current->label = haplo->label;
            hh->current->prob = haplo->prob;
            hh->current->length = haplo->length;
            hh->next = NULL;
         }
         hi = hi->next;
      }
   }

   return 0;
}

/* allocate and initialize cutset for peeling  */
int cutset_init (int l, BGcutnode * cutset)
{
   int   set_start, set_size, i;

   set_start = mrks[l].idxnode;
   set_size = mrks[l].countnode;

   cutset = malloc (set_size * sizeof (BGcutnode));

   for (i = 0; i < set_size; ++i)
   {
      cutset[i].index = set_start + i;
      cutset[i].haps = NULL;
   }

   return (set_size);
}

/*  First we will simply read the DAG file and strip it to six items.         */

int strip_dagfile (void)
{
   FILE *dagfile = NULL;                /* Input dag's file control block.    */
   FILE *oScorF = NULL;                 /* Stripped dag's file control block. */

   char *fname;                         /* The input extra file filename.     */

   char  tmarker[BUFLEN];
   char  buf[BUFLEN];
   char  tallele;
   char  c;

   int   tlevel, tparent, tchild, tcount;
   int   dex, i;

   /* Get the file name of the input extra file.                              */

   fname = get_filent (XTRA)->fname;
   if (fname == NULL)
      give_up (ERD_ixf);

   /* Unzip the input extra file if it is gzipped.                            */

   if (gunzipit (fname))
      give_up (ERD_guz);

   /* Open the input extra file for reading the beagledag input dagfile.      */

   if (dagfile == NULL)
      if (!(dagfile = open_fil (XTRA)))
         give_up (ERR_qit);

   /* Open the output scores file to write the stripped dag file to.          */

   if (oScorF == NULL)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   dex = 0;
   fgets (buf, BUFLEN, dagfile);        /* skip header */
   while (fgets (buf, BUFLEN, dagfile))
   {
      if (buf[0] == '\n')
         continue;                      /* empty line */

      /* Did we finish the line?  If not, scan ahead to the next line.        */
      for (i = 0; i < BUFLEN; i++)
         if (buf[i] == '\n')
            break;

      if (i == BUFLEN)
         while ((c = getc (dagfile), c != '\n' && c != EOF));

      sscanf (buf, "%d %s %d %d %c %d",
              &tlevel, tmarker, &tparent, &tchild, &tallele, &tcount);

      fprintf (oScorF, "%d %d %d %d %c %d\n",
               dex, tlevel, tparent, tchild, tallele, tcount);
      ++dex;
   }
   numedges = dex;
   nummrks = tlevel + 1;

   fclose (dagfile);
   fclose (oScorF);

   return 0;
}

int setup_dag (void)
{
   FILTAB_PTR oScorF_ptb;               /* File table pointer of OSCORE file. */
   FILE *newdag;                        /* Stripped dag's file control block. */

   int   i, ii, l, mx, nn;
   int   ixe, ixn, ice, icn;
   int   tlevel, tparent, tchild, tcount, iallele;

   /* Open the output scores file to use for reading the stripped dagfile.    */

   oScorF_ptb = get_filent (OSCORE);    /* Stripped dag's file table pointer. */
   if (!(newdag = fopen (oScorF_ptb->fname, "r")))
      give_up (ERR_qit);

   /* Set the marker edge indices.                                            */

   edges = malloc (numedges * sizeof (BGedge));
   mrks = malloc (nummrks * sizeof (BGmarker));

/* read in the edges list and set elements
 * note parent and child indicies will be parsed later
 */

   for (i = 0; i < numedges; ++i)
   {
      fscanf (newdag, "%*d %d %d %d %d %d",
              &tlevel, &tparent, &tchild, &iallele, &tcount);
      edges[i].index = i;
      edges[i].allele = iallele;
      edges[i].level = tlevel;
      edges[i].count = tcount;
      edges[i].parent = tparent;
      edges[i].child = tchild;
   }
   fclose (newdag);

/* Now set marker starts and counts of edges */
   i = 0;                               /* count through edges */
   l = 0;                               /* current level       */
   nn = 0;                              /* count of edges for this marker */
   mrks[0].idxedge = 0;

   while (i < numedges)
   {
      while (i < numedges && edges[i].level == l)
      {
         ++i;
         ++nn;
      }
      mrks[l].countedge = nn;
      ++l;
      if (l < nummrks)
         mrks[l].idxedge = i;
      nn = 0;
   }
   mrks[nummrks - 1].countedge = numedges - 2 - mrks[nummrks - 2].idxedge;

/*
     for (i=0; i< nummrks; ++i)
        printf("  %d %d %d\n",  i, mrks[i].idxedge, mrks[i].countedge);
 */

/*  Now go through again  constructing the node lists and counts */
/*  The number of nodes at next level is number of distinct child nodes */
/*  The number of edges  should be  <= 2* # parent nodes      */

/* first count the total number of nodes  */
/* Set the marker edge and node counts at the same time  */

   nn = 0;                              /*  node count -- origin node  */

   for (l = 0; l < nummrks; ++l)        /* count through markers  */
   {
      mx = 0;
      for (i = mrks[l].idxedge; i < mrks[l].idxedge + mrks[l].countedge; ++i)
         if (edges[i].parent > mx)
            mx = edges[i].parent;
      mrks[l].idxnode = nn;
      mrks[l].countnode = mx + 1;
      nn += (mx + 1);
   }

   numnodes = nn;
   printf (" Total number of nodes is %d\n\n", numnodes);

/* and now through one more time to set node structures
 *    and to rest node-indexing in edges list
 */

   nodes = malloc (numnodes * sizeof (BGnode));

   for (l = 0; l < nummrks; ++l)        /* count through markers  */
   {
      ixe = mrks[l].idxedge;            /* starting edge index for marker */
      ixn = mrks[l].idxnode;            /* starting node index for marker */
      ice = mrks[l].countedge;
      icn = mrks[l].countnode;

      for (i = ixe; i < ixe + ice; ++i) /* go through edges  */
      {
         edges[i].parent += ixn;
         edges[i].child += ixn + icn;
      }

      for (i = ixn; i < ixn + icn; ++i) /* go through nodes   */
      {
         nodes[i].index = i;
         nodes[i].parent = l - 1;
         nodes[i].child = l;
         nodes[i].startedge = -1;
         nodes[i].count = 0;
         for (ii = ixe; ii < (ixe + ice); ++ii)
            if (edges[ii].parent == i)
            {
               nodes[i].count += edges[ii].count;
               if (nodes[i].startedge == -1)
                  nodes[i].startedge = ii;
            }
      }
   }

/*   Finally, count through markers again, setting probabilities  */
/*  Total number of haplotypes is nodes[0].count   */
/*  Also set the allele frequencies    */

   for (l = 0; l < nummrks; ++l)        /* count through markers  */
   {
      ixe = mrks[l].idxedge;            /* starting edge index for marker */
      ixn = mrks[l].idxnode;            /* starting node index for marker */
      ice = mrks[l].countedge;
      icn = mrks[l].countnode;
      mrks[l].freq = 0.0;
      printf (" mrk %d %d %d %d %d\n", l, ixe, ice, ixn, icn);

      for (i = ixn; i < ixn + icn; ++i) /* go through nodes   */
      {
         nodes[i].prob = ((double) nodes[i].count) / nodes[0].count;
         printf (" nodes             %d %d %d %d %7.4f\n",
                 nodes[i].index, nodes[i].parent, nodes[i].startedge,
                 nodes[i].count, nodes[i].prob);
      }
      for (i = ixe; i < ixe + ice; ++i) /* go through edges  */
      {
         ii = edges[i].parent;
         edges[i].prob = ((double) edges[i].count) / nodes[ii].count;
         if (edges[i].allele == 1)
            mrks[l].freq += edges[i].count;
         printf ("edge    %d %d %d %d %d %d  %7.4f\n", i, edges[i].level,
                 edges[i].parent, edges[i].child, edges[i].allele,
                 edges[i].count, edges[i].prob);
      }
      mrks[l].freq = mrks[l].freq / nodes[0].count;
      printf ("    freq  %8.4f\n", mrks[l].freq);
   }

   return 0;
}

int setup_haplist (Haplist * nulhaps)
{
   nulhaps->current = (Hap *) malloc (sizeof (Hap));
   nulhaps->current->length = -1;
   nulhaps->current->label = 0;
   nulhaps->current->prob = 0.0;
   nulhaps->next = NULL;

   return 0;
}

int setup_cutset (int ll, int nn, BGcutnode * cutset)
/* set up a cutset of nn nodes starting at locus ll  */
{
   int   k;

   for (k = 0; k < nn; ++k)
   {
      cutset[k].index = k + mrks[ll].idxnode;
      cutset[k].haps = (Haplist *) malloc (sizeof (Haplist));
      setup_haplist (cutset[k].haps);
   }

   return 0;
}

void free_cutset (int nn, BGcutnode * cutset)
{
   int   k;
   Haplist *pcurr, *pnext;

   if (!cutset)
      return;

   for (k = 0; k < nn; ++k)
   {
      pnext = cutset[k].haps;
      free ((void *) pnext->current);
      cutset[k].haps = NULL;
      while (pnext)
      {
         pcurr = pnext;
         pnext = pcurr->next;
         pcurr->current = NULL;
         pcurr->next = NULL;
         free ((void *) pcurr);
      }
   }

   free ((void *) cutset);
   cutset = NULL;

   return;
}

int init_first_cutset (int l, int nn, BGcutnode * cutset)
/* initialize the probabilites of the nodes at level l in the nul haps */
{
   int   k;
   Haplist *iptr;

   for (k = 0; k < nn; ++k)
   {
      iptr = cutset[k].haps;
      while (iptr)
      {
         iptr->current->prob = nodes[cutset[k].index].prob;
         iptr->current->length = 0;
         iptr = iptr->next;
      }
   }

   return 0;
}

int beagledag (int argc, char **argv)
{
   BGcutnode *out_cutset;
   BGcutnode *in_cutset;

   int   start_mrk;
   int   nloc;
   int   l;

   /* ................................  SET-UP   ............................ */

   setupBasic ("", argc, argv, PNAM);
   fflush (stdout);

   /* Make local copies of the parameter statement values for beagledag.      */

   strip_dagfile ();

   printf ("\n Number of edges = %d\n", numedges);
   printf (" Number of markers = %d\n", nummrks);

   /* Set up dag structure of nodes, markers, edges.                          */
   setup_dag ();

   /* Set up (and/or later clean out) cutsets.                                */
   /* Initialize cutset at starting marker.                                   */

   start_mrk = 130;                     /* testing */
   nloc = 10;
   haplo = (Hap *) malloc (sizeof (Hap)); /* Set up a working hap.            */

   printf (" Starting cutset setup\n");
   fflush (stdout);
   in_cutset = malloc (mrks[start_mrk].countnode * sizeof (BGcutnode));
   setup_cutset (start_mrk, mrks[start_mrk].countnode, in_cutset);

   printf (" Starting init cutset\n");
   fflush (stdout);
   init_first_cutset (start_mrk, mrks[start_mrk].countnode, in_cutset);

   /* Peel across the cutsets of the specified marker range.                  */

   for (l = start_mrk; l < start_mrk + nloc; ++l)
   {
      printf (" starting set output cutset mrk %d\n", l);
      fflush (stdout);
      out_cutset = malloc (mrks[l + 1].countnode * sizeof (BGcutnode));
      setup_cutset (l + 1, mrks[l + 1].countnode, out_cutset);
      peel_marker (l, in_cutset, out_cutset);
      free_cutset (mrks[l].countnode, in_cutset);
      in_cutset = out_cutset;
   }

   /* Pool the haplotypes at the final cutset.                                */

   printf ("\n Starting final cutset pooling\n");
   out_cutset = malloc (sizeof (BGcutnode));
   out_cutset->index = 0;
   out_cutset->haps = (Haplist *) malloc (sizeof (Haplist));
   setup_haplist (out_cutset->haps);
   pool_hapsets (mrks[start_mrk + nloc].countnode, in_cutset, out_cutset);

   printf (" Done!\n");

   /* We are done, so free up the storage that has been allocated by Morgan   */
   /* to support the beaglesim subroutine.                                    */

   free_fnams ();

   return (0);
}
