/******************************************************************************/
/*                                                                            */
/* Created from the fgl2ibd.c file.                                           */
/*                                                                            */
/******************************************************************************/

#define PNAM "ibd_create"

#include "parsie.h"
#include "ibd_create.h"

/******************************************************************************/
/*                                                                            */
/* The fgl2dgl subprogram main routine.                                       */
/*                                                                            */
/******************************************************************************/

int fgl2dgl (int argc, char **argv)
{
   FILE *sim_chroms = NULL;
   FILE *oScorF = NULL;

   char *input_file;

   /* ................................  SET-UP   ............................ */

   setupBasic ("A", argc, argv, PNAM);

   /* Tell the user what scale factor for the conversion of centimorgans to   */
   /* million base pairs is being used in this run of fgl2dgl.                */

   printf (" The scale factor used for this run of fgl2dgl is");
   printf (" %7.4f Mbp per cM\n\n", MBPperCM);

   /* Get the file name of the input ibd_graph gamete data file.              */

   input_file = get_filent (IGAM)->fname;
   if (input_file == NULL)
      give_up (ERD_igd);

   /* Open the Morgan input gamete data file to use as fgl2dgl's input        */
   /* ibd_graph gamete data file, with the simulated FGL's included.          */

   if (sim_chroms == NULL)
      if (!(sim_chroms = open_fil (IGAM)))
         give_up (ERR_qit);

   /* Open the output scores file to use as fgl2dgl's results file.           */

   if (oScorF == NULL)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   /* Terminal output for fgl2dgl is improved by outputting an extra line.    */

   fprintf (stdout, "\n");
   fflush (stdout);

   /* Call init_snp_pos() to intialize the SNP positions vector, snp_pos.     */

   init_snp_pos (NMarks);

   /* Read in ALL the input ibd_graph gamete data from simpop_fgl's output.   */

   read_gamete_data (sim_chroms, &num_gametes, &ibdgraph_gametes);
   fclose (sim_chroms);

   if (DebugLevel >= 3)
   {
      /* Output the selected marker numbers and their corresponding positions */
      /* in micro centimorgans when fgl2dgl is being debugged at level 3 or   */
      /* higher.                                                              */

      prnt_marker_positions ();
   }

   /* Call the make_dgl_graphs_file subroutine to write the ibd_graphs to the */
   /* output scores file.                                                     */

   make_dgl_graphs_file (oScorF, num_gametes, ibdgraph_gametes);

   /* Close the output scores file.                                           */

   fclose (oScorF);

   /* We are done, so free up the storage that has been allocated by the      */
   /* fgl2dgl subprogram.                                                     */

   free_gamete_data (num_gametes, ibdgraph_gametes);

   free_marek ();
   free_auto ();
   free_locnams ();
   free_fnams ();

   /* Free the remaining fgl2dgl subprogram allocated memory.                 */

   free_1d ((void *) &snp_pos);

   return 0;
}

/******************************************************************************/
/*                                                                            */
/* The make_dgl_graphs_file subroutine is called to output to the file passed */
/* as the first argument, the FGL's passed in the third argument.  The number */
/* of gametes in the vector passed as the third argument needs to be passed   */
/* as the second argument.                                                    */
/*                                                                            */
/* The output produced is in a compact format.  This subroutine outputs one   */
/* line for each gamete passed in the third argument consisting of the        */
/* selected individual's name, this individual's last tloc and first marker   */
/* FGL's, the number of changes for this individual and for each change the   */
/* marker number at which the change occurred followed by its FGL.            */
/*                                                                            */
/******************************************************************************/

void make_dgl_graphs_file (FILE * fptr, int num_gams, Gamete * gametes)
{
   Chromseglist *first_seglist;         /* First marker gamete segment list.  */
   Chromseglist *curr_seglist;          /* Current gamete segment list.       */
   Chromseglist *next_seglist;          /* Next gamete segment list.          */

   int   fncp_rc;                       /* Most recent return code from the   */
   /*                                      find_next_change_point subroutine. */

   int   curr_mark_ndx;                 /* Current marker subscript.          */
   int   next_mark_ndx;                 /* Next marker subscript.             */

   int   ch_cnt;                        /* Count of number of changes.        */

   int   first_fgl;                     /* First marker founder genome label. */
   int   curr_fgl;                      /* Current founder genome label.      */

   int   g1;                            /* Current subscript into the vector  */
   /*                                      of input ibd_graph gametes.        */

   /* Perform the following loop once for each gamete in the vector of input  */
   /* ibd_graph gametes.                                                      */

   for (g1 = 0; g1 < num_gams; ++g1)
   {
      if (DebugLevel >= 3)
      {
         /* For debugging purposes, print the contents of this gamete's FGL   */
         /* segment list.                                                     */

         prnt_gam_segs (gametes[g1]);
      }

      /* Find the FGL segment containing the first selected marker.           */

      curr_seglist = gametes[g1].seglist;
      curr_mark_ndx = UNDEF;

      fncp_rc = find_next_change_point (curr_seglist, curr_mark_ndx,
                                        &first_seglist, &next_mark_ndx);

      /* Output the individual's name from the gamete, a 0 placeholder for    */
      /* the tloc's FGL and the first marker's FGL.                           */

      first_fgl = first_seglist->current->fgl;
      fprintf (fptr, "%s %d %d", gametes[g1].name, 0, first_fgl);

      /* Count the number of change points for the current (g1'st) gamete.    */

      curr_seglist = first_seglist;
      curr_mark_ndx = 0;
      ch_cnt = 0;

      while (curr_seglist != NULL && fncp_rc == TRUE)
      {
         fncp_rc = find_next_change_point (curr_seglist, curr_mark_ndx,
                                           &next_seglist, &next_mark_ndx);

         if (next_seglist != NULL)
            ++ch_cnt;

         curr_seglist = next_seglist;
         curr_mark_ndx = next_mark_ndx;
      }

      /* Output the number of change points found for the current gamete.     */

      fprintf (fptr, " %d", ch_cnt);
      fflush (fptr);

      /* Go back to the first marker gamete segment list to now find each     */
      /* change point.                                                        */

      curr_seglist = first_seglist;
      curr_mark_ndx = 0;
      fncp_rc = TRUE;

      /* For each change point, output the marker number at which the change  */
      /* occurred followed by its FGL.                                        */

      while (curr_seglist != NULL && fncp_rc == TRUE)
      {
         fncp_rc = find_next_change_point (curr_seglist, curr_mark_ndx,
                                           &next_seglist, &next_mark_ndx);

         if (next_seglist != NULL)
         {
            curr_fgl = next_seglist->current->fgl;
            fprintf (fptr, " %d %d", MarSel[next_mark_ndx], curr_fgl);
         }

         curr_seglist = next_seglist;
         curr_mark_ndx = next_mark_ndx;
      }

      fprintf (fptr, "\n");
      fflush (fptr);
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* The find_next_change_point subroutine is called to locate the "next change */
/* point."  The first two arguments are input to the subroutine telling it    */
/* what the previously used segment list pointer and previously used selected */
/* marker subscript were.  This subroutine continues from the input locations */
/* trying to find the next change point along the current gamete.  When a     */
/* change point is found, this subroutine returns a pointer to the segment    */
/* list pointer block and selected marker number of the change point marker.  */
/* The calling routine can find the FGL being changed to in the segment list  */
/* pointer block.                                                             */
/*                                                                            */
/* However there is a special treatment provided by the subroutine if the     */
/* caller wants to find the segment list block for the first selected marker  */
/* in a new gamete.  If called with prev_mark_ndx < 0 then this subroutine is */
/* being called to find the segment list containing the first selected marker */
/* rather than the next change point.  This was added so that this subroutine */
/* would have a way to initialize its variables when a new gamete is being    */
/* worked on.                                                                 */
/*                                                                            */
/* This subroutine returns a TRUE return code if there is a possibility that  */
/* additional change points further along the gamete currently being worked   */
/* on.  This subroutine returns a FALSE return code if it is impossible for   */
/* there to be any further change points along the gamete currently being     */
/* worked on.  These return codes should be used by the calling routine to    */
/* decide whether find_next_change_point should be called again while working */
/* along the current gamete.                                                  */
/*                                                                            */
/******************************************************************************/

int find_next_change_point (const Chromseglist * prev_seglist,
                            const int prev_mark_ndx,
                            Chromseglist ** next_seglist, int *next_mark_ndx)
{
   Chromseglist *curr_seglist;          /* Current gamete segment list.       */

   int   curr_before_selected;          /* This is a binary flag set to TRUE  */
   /*                                      so long as the end position of the */
   /*                                      current segment is less than the   */
   /*                                      position of the current selected   */
   /*                                      marker.                            */

   int   next_sel_pos;                  /* Position of next selected marker.  */

   int   curr_pos;                      /* Position of current marker.        */

   int   prev_fgl;                      /* Previous founder genome label.     */
   int   curr_fgl;                      /* Current founder genome label.      */

   int   mark_ndx;                      /* Current selected marker subscript. */

   /* If prev_seglist == NULL, indicate to the caller that no additional      */
   /* change points can possibly be found along the current gamete.           */

   if (prev_seglist == NULL)
   {
      *next_seglist = NULL;
      return FALSE;
   }

   /* Check to see if this is the first call of find_next_change_point for    */
   /* the current gamete.                                                     */

   if (prev_mark_ndx < 0)
   {
      /* If this is the first call, then find the FGL segment containing the  */
      /* first selected marker.  The caller can get the FGL for this segment  */
      /* from the current gamete segment list structure.                      */

      next_sel_pos = snp_pos[0];

      curr_seglist = (Chromseglist *) prev_seglist;
      curr_before_selected = TRUE;

      while (curr_before_selected == TRUE)
      {
         curr_pos = curr_seglist->current->endpos;
         curr_before_selected = (curr_pos < next_sel_pos);
         if (curr_before_selected == TRUE)
            curr_seglist = curr_seglist->next;
      }

      *next_seglist = curr_seglist;
      *next_mark_ndx = 0;
      return TRUE;
   }

   /* At this point we know that prev_seglist is not equal to NULL because if */
   /* it had been equal to NULL we would have already returned to the caller. */

   curr_seglist = (Chromseglist *) prev_seglist;
   prev_fgl = curr_seglist->current->fgl;

   for (mark_ndx = (prev_mark_ndx + 1); mark_ndx < NMarks; ++mark_ndx)
   {
      next_sel_pos = snp_pos[mark_ndx];

      curr_before_selected = TRUE;

      while (curr_before_selected == TRUE)
      {
         if (curr_seglist != NULL)
         {
            curr_pos = curr_seglist->current->endpos;
            curr_before_selected = (curr_pos < next_sel_pos);
         }
         else
            curr_before_selected = FALSE;

         /* If the current segment list end position is still less than the   */
         /* position of the selected marker continue looking for a segment    */
         /* ending past the selected marker's position.                       */

         if (curr_before_selected == TRUE)
            curr_seglist = curr_seglist->next;

         else
         {
            /* We may have located a change point.  This would really be a    */
            /* change point if the two FGL's are not equal.                   */

            if (curr_seglist != NULL)
               curr_fgl = curr_seglist->current->fgl;
            else
               curr_fgl = UNDEF;

            if (prev_fgl != curr_fgl)
            {
               /* This is a real change point between the selected markers.   */

               *next_seglist = curr_seglist;
               *next_mark_ndx = mark_ndx;

               /* If this is the last selected marker or if there are no more */
               /* IBD segments, then there can't be any further change points */
               /* on the current gamete.                                      */

               if (((mark_ndx + 1) >= NMarks) || curr_seglist == NULL)
                  return FALSE;
               else
                  return TRUE;
            }
            else
               continue;                /* Check the next selected marker.    */
         }
      }
   }

   *next_seglist = NULL;

   return FALSE;
}
