/******************************************************************************/
/*                                                                            */
/* Created from create.c.                                                     */
/* simpop_fgl output -> population ibd states.                                */
/*                                                                            */
/******************************************************************************/

#define PNAM "ibd_create"

#include "markers.h"
#include "parsie.h"

#include "ibd_haplo_directory.h"
#include "ibd_create.h"

/******************************************************************************/
/*                                                                            */
/* The fgl2ibd subprogram.                                                    */
/*                                                                            */
/******************************************************************************/

int fgl2ibd (int argc, char **argv)
{
   FILE *sim_chroms = NULL;
   FILE *oScorF = NULL;

   FILTAB_PTR oXtra_pft;
   FILE *oXtra = NULL;

   int   i1;                            /* Subscript of the first individual  */
   /*                                      in a pair of individuals.          */

   int   i2;                            /* Subscript of the second individual */
   /*                                      in a pair of individuals.          */

   int   c1 = 0;                        /* Component subscript (always == 0). */

   /* ................................  SET-UP   ............................ */

   setupBasic ("A", argc, argv, PNAM);

   if (DoingAllPairs)
   {
      /* Call alloc_for_pairs to allocate storage for all pairs case.         */
      alloc_for_pairs ();
   }

   /* Tell the user what scale factor for the conversion of centimorgans to   */
   /* million base pairs is being used in this run of fgl2ibd.                */

   printf (" The scale factor used for this run of fgl2ibd is");
   printf (" %7.4f Mbp per cM\n\n", MBPperCM);

   /* Open the Morgan input gamete data file to use as fgl2ibd's input        */
   /* ibd_graph gamete data file, with the simulated FGL's included.          */

   if (sim_chroms == NULL)
      if (!(sim_chroms = open_fil (IGAM)))
         give_up (ERR_qit);

   /* Open the output scores file to use as fgl2ibd's results file.           */

   if (oScorF == NULL)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   /* If the user has named an output extra file, open it to use as fgl2ibd's */
   /* score set identification's file.                                        */

   oXtra_pft = get_filent (OXTR);
   if (oXtra_pft != NULL && oXtra_pft->fname)
      if (!(oXtra = open_fil (OXTR)))
         give_up (ERR_qit);

   /* Terminal output for fgl2ibd is improved by outputting an extra line.    */

   fprintf (stdout, "\n");
   fflush (stdout);

   /* If the user has not provided any proband gamete statements, then there  */
   /* is nothing for the fgl2ibd subprogram to do.                            */

   if (!DoingAllPairs && (NPrGam == NULL || NPrGam[c1] == NULL))
   {
      fprintf (stderr, "\n The fgl2ibd subprogram is terminating because no");
      fprintf (stderr, " proband gametes were specified  (Q)\n\n");

      return 5;
   }

   /* Call init_snp_pos() to intialize the SNP positions vector, snp_pos.     */

   init_snp_pos (NMarks);

   /* Read in ALL the input ibd_graph gamete data from simpop_fgl's output.   */

   read_gamete_data (sim_chroms, &num_gametes, &ibdgraph_gametes);
   fclose (sim_chroms);

   /* We need to have the input ibd_graph gamete data sorted into ascending   */
   /* order (as defined by the comparGams subroutine).                        */

   qsort (ibdgraph_gametes, num_gametes, sizeof (Gamete), comparGams);

   /* Call the init_ss_ibd_data subroutine to set up the ss_ibd_data vector.  */

   init_ss_ibd_data (oScorF);

   /* Call the make_names_vector subroutine to initialize the indiv_names     */
   /* vector to contain a sorted vector of the unique names found in the      */
   /* input ibd_graph gamete data file.                                       */

   make_names_vector ();

   /* Initialize the four output lengths variables.                           */

   init_ibd_lengths ();

   /* The gen_fgl2ibd_labels subroutine is called to execute the main loop in */
   /* fgl2ibd.  This subroutine generates and outputs the IBD state label for */
   /* each score set indicated by the fgl2ibd user.                           */

   if (DoingAllPairs)
   {
      for (i1 = 0; i1 < num_names; ++i1)
      {
         for (i2 = (i1 + 1); i2 < num_names; ++i2)
         {
            /* Call the set_for_pair subroutine to set up the global proband  */
            /* gamete variables for the given pair of individuals.            */

            set_for_pair (i1, i2);

            gen_fgl2ibd_labels (i1, i2, oScorF, oXtra);
            if (i1 < num_names - 2)
               fprintf (stdout, "\n");

            ++ScSetNo[0][0];
         }
      }
   }
   else
      gen_fgl2ibd_labels (UNDEF, UNDEF, oScorF, oXtra);

   /* Close the scores file and output extra file.                            */

   fflush (oScorF);
   fclose (oScorF);

   if (oXtra)
   {
      fflush (oXtra);
      fclose (oXtra);
   }

   /* We are done, so free up the storage that has been allocated by the      */
   /* fgl2ibd subprogram.                                                     */

   free_gamete_data (num_gametes, ibdgraph_gametes);
   free_ss_ibd_data ();
   free_marek ();
   free_auto ();
   free_locnams ();
   free_fnams ();

   /* Free the remaining fgl2ibd subprogram allocated memory.                 */

   free_2d ((void *) &indiv_names);
   free_1d ((void *) &snp_pos);

   return 0;
}

/******************************************************************************/
/*                                                                            */
/* The gen_fgl2ibd_labels subroutine is the primary subroutine called by the  */
/* fgl2ibd subprogram.  It generates and outputs IBD state labels for all the */
/* score sets provided by the caller.                                         */
/*                                                                            */
/******************************************************************************/

void gen_fgl2ibd_labels (int indiv1, int indiv2, FILE * oScorF, FILE * oXtra)
{

   Gamete *current_gamete;              /* Pointer to the just located entry  */
   /*                                      in the input ibd_graph gamete data */
   /*                                      array.                             */

   Gamete key_gamete;                   /* A locally defined Gamete array     */
   /*                                      element structure to pass to the   */
   /*                                      bsearch subroutine.                */

   double output_snp_pos;               /* For the current marker temporarily */
   /*                                      holds the marker position (in Cm). */

   char  pat_format[4];                 /* Format used for pattern output.    */

   char *gamete_id;                     /* Current proband gamete identifier. */
   int   l_gamete_id;                   /* Length of current PG identfier.    */

   int **FGLs;                          /* Matrix of FGLs at markers for each */
   /*                                      scored gamete.                     */
   int   nf;                            /* Character length of longest FGL.   */

   int   pattern[10];                   /* State spec for up to 10 gametes.   */

   int   ss_size;                       /* Size of the current score set.     */
   int   ss_size_ndx;                   /* Current score set size subscript.  */

   int   lm_auto_label;                 /* The IBD state label output by the  */
   /*                                      lm_auto program.                   */
   int   state_ndx;                     /* Subscript to the current marker's  */
   /*                                      IBD state.                         */

   int   c1 = 0;                        /* Component subscript (always == 0). */
   int   s1;                            /* Subscript to current score set.    */
   int   g1;                            /* Subscript to proband gamete.       */
   int   m1;                            /* Subscript to current marker.       */

   /* Allocate storage for the FGL's array.                                   */

   FGLs = alloc_int2d (MxGam, NMarks);

   /* I don't believe these 2 initialization statements are really needed.    */
   /* I just put them in so that when I looked at key_gamete in gdb, these    */
   /* two components would not just display utter garbage.                    */

   key_gamete.index = 0;
   key_gamete.seglist = NULL;

   /* In the main loop for the fgl2ibd subprogam generate and output the IBD  */
   /* state label for each score set indicated by the fgl2ibd user.           */

   for (s1 = 0; s1 < NScSet[c1]; ++s1)  /* For each score set:                */
   {
      ss_size = NPrGam[c1][s1];         /* Get size of the current score set. */
      ss_size_ndx = ss_size - 1;        /* Set the score set size subscript.  */

      printf (" === Start of the computation of IBD state labels for score");
      printf (" set %*d ===\n", nsn, ScSetNo[c1][s1]);

      /* Inform the user which ordering of states will be used for the IBD    */
      /* state indices output for the current score set in fgl2ibd's state    */
      /* labels output results file.                                          */

      printf ("     IBD state indices output for this score set are in");
      if (Lexico == N && ss_size == 4)
         printf (" traditional (Jacquard)");
      else
         printf (" lexicographical");
      printf (" order\n");

      fflush (stdout);

      /* If the user has named an output extra file, then output the score    */
      /* set identification information to the output extra file, 1 line per  */
      /* score set.                                                           */

      if (oXtra)
      {
         fprintf (oXtra, " %*d", nsn, ScSetNo[c1][s1]);
         fprintf (oXtra, " %2d", NPrGam[c1][s1]);
         fprintf (oXtra, " %*d", nst, PTgg[ss_size]);

         for (g1 = 0; g1 < ss_size; ++g1)
         {
            fprintf (oXtra, "  %*s", nc, ProbNam[c1][s1][g1]);
            fprintf (oXtra, " %d", ProbGam[c1][s1][g1]);
         }

         fprintf (oXtra, "\n");
         fflush (oXtra);
      }

      /* For each proband gamete provided by the user, we need to calculate   */
      /* and then print the IBD states along the chromosome.                  */

      for (g1 = 0; g1 < ss_size; ++g1)
      {
         /* Prepare a key_gamete Gamete array element structure and a current */
         /* proband gamete identifier for use throughout this loop.           */

         key_gamete.name = ProbNam[c1][s1][g1];
         key_gamete.which = ProbGam[c1][s1][g1];

         /* Include quotes along with proband gamete name, which (maternal or */
         /* paternal) gamete in the current proband gamete identifier and     */
         /* space for the NULL character at the end of the string.            */

         l_gamete_id = strlen (key_gamete.name) + 5;
         gamete_id = malloc (l_gamete_id * sizeof (char));
         alloc_chk (WHERE, gamete_id);

         sprintf (gamete_id, "\"%s %d\"", key_gamete.name, key_gamete.which);

         /* Locate the chosen proband gamete within the array of gametes.     */

         current_gamete = bsearch (&key_gamete, ibdgraph_gametes, num_gametes,
                                   sizeof (Gamete), comparGams);

         /* Check that the selected proband gamete was found in the input     */
         /* ibd_graph gamete data vector.                                     */

         if (current_gamete == NULL)
         {
            fprintf (stderr, "\n Proband gamete %s was not found", gamete_id);
            fprintf (stderr, " in the input ibd_graph gamete data\n\n");
            continue;
         }

         /* Calculate the IBD states along the chromosome.                    */

         nf = fgl_marks (current_gamete->seglist, NMarks, snp_pos, FGLs[g1]);

         if (DebugLevel >= 3)
         {
            fprintf (oScorF, "\n Gamete %s", gamete_id);
            for (m1 = 0; m1 < NMarks; ++m1)
               fprintf (oScorF, " %*d", nf, FGLs[g1][m1]);
         }

         free (gamete_id);
      }

      if (DebugLevel >= 3)
      {
         fprintf (oScorF, "\n\n");
         fflush (oScorF);
      }

      /* Now output the IBD state labels.                                     */

      for (m1 = 0; m1 < NMarks; ++m1)
      {
         output_snp_pos = FirstMarkPos + MarkPos[0][m1];
         ibd_state (FGLs, m1, ss_size, pattern);
         lm_auto_label = label_state (ss_size, pattern);
         state_ndx = ss_ibd_data[ss_size_ndx].index[lm_auto_label];

         /* When lexicographical output order is not wanted and if the number */
         /* of gametes in the current score set is 4, we want the output IBD  */
         /* state index to be converted from a lexicographical IBD state      */
         /* index to a Jacquard IBD state index.                              */

         if (Lexico == N && ss_size == 4)
            state_ndx = jacquard_index[state_ndx];

         fprintf (oScorF, " %*d ", nsn, ScSetNo[c1][s1]);
         fprintf (oScorF, " %*d ", nm, MarSel[m1]);
         fprintf (oScorF, " %10.6f ", output_snp_pos);
         fprintf (oScorF, " %*d ", nst, state_ndx + 1);
         fprintf (oScorF, " %*d ", nst, lm_auto_label);

         /* Include the IBD state pattern in the oScorF.  If debugging this   */
         /* subprogram, don't include any spaces within the pattern.          */

         if (DebugLevel > 0)
         {
            strcpy (pat_format, "%1d");
            fprintf (oScorF, " ");
         }
         else
            strcpy (pat_format, " %d");

         for (g1 = 0; g1 < ss_size; ++g1)
            fprintf (oScorF, pat_format, pattern[g1]);

         fprintf (oScorF, "\n");
         fflush (oScorF);
      }

      fprintf (stdout, "\n");
      if (s1 < NScSet[c1] - 1)          /* Unless this is the last score set, */
         fprintf (stdout, "\n");        /* print another blank line.          */
      fflush (stdout);
   }

   /* Free the remaining gen_fgl2ibd_labels subroutine allocated memory.      */

   free_2d ((void *) &FGLs);

   return;
}
