/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/******************************************************************************/
/*                                                                            */
/* Traits library.                                             EAT March 02   */
/* routines  set_markdat : moved to Traits library March 02                   */
/* plus new routines:                                                         */
/* The following functions were removed, as no longer compatible              */
/* with parameter statements and output from Pars/ functions:                 */
/*   reset_traitmap  -- resets Loc0 and Rec0   for lm_auto                    */
/*   unset_recval    -- recovers original input recombs and proportions       */
/*   reset_recval    -- resets RecVal and IntVal for lm_lods and other progs  */
/*                                                             MAJ  Feb 04    */
/* Modified to put geno-data into nghds structures             EAT  Sep 04    */
/*   genotypic trait data will now also be set here-- not in set_trait_dat    */
/*                                                                            */
/******************************************************************************/

#include "progSimSel.h"                 /* Declares structure psx points to.  */
#define BT_LODS     (psx->pcod == 'o')

#include "traits.h"

/* Copies marker data to nghd[].g_dat.                                        */

void set_markdat (int n_marks, int n_compnts)
{
   int  *gobs;                          /* Pointer to an array of 0's and 1's */
   /*                                      where 0 -> the corresponding       */
   /*                                      member of the pedigree has no      */
   /*                                      observed marker or trait data and  */
   /*                                      1 -> there is observed data.       */

   int   ngobs;                         /* Number of pedigree members with    */
   /*                                      any observed data.                 */

   int   g_ndx;                         /* Subscript into NghdGenoData of the */
   /*                                      current array of genotype data.    */

   int   m_ndx;                         /* Offset to first marker in g_dat[]. */

   int   comp_no;                       /* Current component number.          */
   int   cfound;                        /* Set to 1 if any marker or trait    */
   /*                                      data for the current component was */
   /*                                      found.                             */

   int   i1, i2;

   /* Allocate a vector of integers initialized to 0 (for UNOBSERVED).  These */
   /* integers need to be changed to 1 (for OBSERVED) for each individual in  */
   /* the pedigree who has observed marker data and/or observed trait data.   */

   gobs = calloc (pedsize, sizeof (int));
   alloc_chk (WHERE, gobs);

   ngobs = 0;

   /* First we need to count how many individuals have either observed marker */
   /* or observed genotypic trait data.                                       */

   /* There may be individuals counted in NObs, for which in fact the marker  */
   /* data are 0.                                                             */

   for (i1 = 0; i1 < NObs; ++i1)
   {
      ObsIx[i1] = get_posit (ObsNam[i1]);
      if (ObsIx[i1] == INVALID_INDEX)
         give_warn (ERA_mdp, ObsNam[i1]);
      else
      {
         for (i2 = 0; i2 < n_marks; ++i2)
         {
            if ((ObsMark[i1][i2][0] + ObsMark[i1][i2][1]) > 0)
               break;
         }
         if (i2 < n_marks)
         {
            gobs[ObsIx[i1]] = OBSERVED;
            ++ngobs;
         }
      }
   }

   /* There may be individuals with observed genotypic trait data who are not */
   /* included in the list of individuals with observed marker data.          */

   if (TraitTyp == GENOTYPIC)
      for (i1 = 0; i1 < pedsize; ++i1)
      {
         if ((gobs[i1] == UNOBSERVED) && (data[i1][5] > 0))
         {
            gobs[i1] = OBSERVED;
            ++ngobs;
         }
      }

   /* There is no reason to continue the set_markdat subroutine if there are  */
   /* no individuals in the pedigree with any marker data or any trait data.  */

   if (ngobs == 0)
   {
      free (gobs);
      return;
   }

   /* Allocate storage for storing the genotype data in the elements of the   */
   /* nghd data structure.  calloc is used to initialize this storage to all  */
   /* zeroes.                                                                 */

   NghdGenoData = calloc ((ngobs * NLoci * 2), sizeof (int));
   alloc_chk (WHERE, NghdGenoData);
   g_ndx = 0;

   /* Initialize the offset at which the data should be stored in g_dat.      */

   m_ndx = max (NTLocs, 1) * 2;

   for (i1 = 0; i1 < NObs; ++i1)
   {
      if (ObsIx[i1] != INVALID_INDEX && gobs[ObsIx[i1]] == OBSERVED)
      {
         nghd[ObsIx[i1]].g_dat = NghdGenoData + g_ndx * NLoci * 2;

         for (i2 = 0; i2 < n_marks; ++i2)
         {
            nghd[ObsIx[i1]].g_dat[i2 * 2 + m_ndx] = ObsMark[i1][i2][0];
            nghd[ObsIx[i1]].g_dat[i2 * 2 + m_ndx + 1] = ObsMark[i1][i2][1];
         }
         ++g_ndx;
      }
   }

   if (TraitTyp == GENOTYPIC)
   {
      for (i1 = 0; i1 < pedsize; ++i1)
      {
         if (data[i1][5] > 0)
         {
            if (nghd[i1].g_dat == NULL)
            {
               nghd[i1].g_dat = NghdGenoData + g_ndx * NLoci * 2;
               ++g_ndx;
            }

            if (data[i1][5] == 1)       /* Individual is homozygous normal.   */
            {
               nghd[i1].g_dat[0] = 1;
               nghd[i1].g_dat[1] = 1;
            }
            else if (data[i1][5] == 3)  /* Individual is a recessive carrier. */
            {
               nghd[i1].g_dat[0] = 1;
               nghd[i1].g_dat[1] = 2;
            }
            else if (data[i1][5] == 4)  /* Individual is homozygous affected. */
            {
               nghd[i1].g_dat[0] = 2;
               nghd[i1].g_dat[1] = 2;
            }
         }
      }
   }

   /* When set_markdat is called by any Morgan program other than in the      */
   /* base_trait_lods program, warn the user about each component for which   */
   /* there is no observed marker data.                                       */

   if (!BT_LODS)
   {
      for (comp_no = 1; comp_no <= n_compnts; ++comp_no)
      {
         cfound = FALSE;

         for (i1 = 0; i1 < NObs; ++i1)
         {
            if (ObsIx[i1] != INVALID_INDEX && nghd[ObsIx[i1]].component ==
                comp_no)
               cfound = TRUE;
         }

         if (cfound == FALSE)
            give_warn (ERA_nmd, comp_no);
      }
   }

   free (gobs);

   return;
}
