/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "parsie.h"
#include "markers.h"
#include "rans.h"
#include "quantset.h"

/**
 * Calls functions to do the setup steps for the Morgan programs
 * which use an individuals file.
 * This includes setting up parameters and data structures for the
 * marker and trait model,
 * reading the individuals file and setting up the pedigree structure,
 * storing observed marker and trait data for the individuals.
 */
void setupwithIndivList (int argc, char **argv, char *ppn, int nh)
{
   /* Initialize exit procedure, print headings.
    * fncs:  atexit(), print_head()
    * dirs:  Stuff
    */
   initMorgProg (argv);

   /* Process parameter statements, set up globals resulting from these.
    * fncs:  proc_pars(), prntStatusPP(), proc_auto(), proc_end(),
    *        proc_seeds(), prntStatusCP()
    * vars:  globals in globs.h
    * dirs:  Pars, Stuff
    */
   procParIndivs (argc, argv, ppn);

   /* Check, and massage if necessary, the individuals file.
    * fncs:  chk_indfil(), prntStatusCIF()
    * dirs:  Markers, Pedchk, Stuff
    */
   chk_indfil ();

   /* Read in pedigree data and set up pedigree structure.
    * fncs:  alloc_ped(), open_ind(), input_ped(), partial_nghds()
    * vars:  int data[PedSize][NColData], Ind nghd[PedSize+1]
    *        double quant[NTraits][PedSize], Ind nghd[].q_dat[2*(NMarks+1)]
    * dirs:  Nghds, Quant, Stuff
    */

   if (TraitTyp == QUANTITATIVE)
      quant = allocQPed ();

   setInd (quant);

   /* Set observed marker and trait data for pedigree members.
    * fncs:  set_markdat(), set_traitdat(), prnt_traitdat(),
    *        set_affind(), set_qdat()
    * vars:  Ind nghd[].g_dat[2*(NMarks+1)],
    *        Ind nghd[].d_dat[2*(NMarks+1)]
    * dirs:  Markers
    */
   setObsDat ();

   if (TraitTyp == QUANTITATIVE)
      setQObsDat (quant);

   /* Set up chromosome map for markers and trait.
    * fncs:  alloc_chromo(), set_chromosomes()
    * vars:  int chromo[nh+1][NMarks+1], revchro[nh+1][NMarks+1],
    *        double xecomo[2][nh+1][NMarks+1]
    * dirs:  Markers
    */
   setMaps (nh);

   /* set up type of each locus.
    * fncs:  set_traits(),  print_traits()
    * vars:  Trait traits[NMarks+1]
    * dirs:  Markers
    */
   setLocTyp ();

   return;
}
