/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "nghds.h"

/*
 * Function to form the components of a pedigree after the nghds have been
 * set.  It is assumed on entry that the "component" indicator of each
 * individuals is -1 (as now set by nghds).  An infection subroutine is used
 * to determine connected sets. The component indicator of each individual is
 * set.  The linked list of components, each being a list of individuals, is
 * set up. the total number of components is counted, and returned.
 *
 * Written and installed; 09/1993  (EAT)
 *
 * free_complists() which frees the Compoentns structures updated
 *   and moved here               (01/2005, EAT)
 *
 */

int neighbour (int j, int k)            /* has individual j a neighbour in
                                         * component k */
{
   Ind  *ipt;
   Marlist *mlpt;
   Indlist *ilpt;
   ipt = &nghd[j];
   if (!isNullNode (ipt->pa))
   {
      if (ipt->pa->component == k)
         return (1);
      if (ipt->ma->component == k)
         return (1);
   }
   for (mlpt = ipt->marriages; mlpt != NULL; mlpt = mlpt->next)
   {
      if (mlpt->current->spouse->component == k)
         return (1);
      for (ilpt = mlpt->current->kids; ilpt != NULL; ilpt = ilpt->next)
         if (ilpt->current->component == k)
            return (1);
   }

   return (0);
}

/******************************************************************************/
/*                                                                            */
/* Call the cmplist_init subroutine to initialize the entry in the cmplist    */
/* array for the comp_no indicated by the first argument to this subroutine.  */
/* The number of individuals in the component is the second argument to this  */
/* subroutine.                                                                */
/*                                                                            */
/* It is important that the caller of this subroutine remembers that cmplist  */
/* is subscripted starting at origin 1, not 0, so the first argument needs to */
/* be no less than 1.                                                         */
/*                                                                            */
/* This subroutine sets up a linked list of individuals in the component.  It */
/* also sets component size and founder counts.                               */
/*                                                                            */
/******************************************************************************/

void cmplist_init (int comp_no, int comp_size)
{
   Indlist *iptr;                       /* Current pointer to individual.     */

   int   n1;                            /* Current subscript into nghd.       */

   cmplist[comp_no].index = comp_no;
   cmplist[comp_no].size = comp_size;
   cmplist[comp_no].nfou = 0;
   cmplist[comp_no].naff = 0;
   cmplist[comp_no].nunaff = 0;
   cmplist[comp_no].nmeioses = 0;
   cmplist[comp_no].meioses = NULL;
   cmplist[comp_no].ind_aff = NULL;
   cmplist[comp_no].ind_unaff = NULL;

   cmplist[comp_no].people = (Indlist *) malloc (sizeof (Indlist));
   alloc_chk (WHERE, cmplist[comp_no].people);

   iptr = cmplist[comp_no].people;
   iptr->current = (Ind *) malloc (sizeof (Ind));
   alloc_chk (WHERE, iptr->current);

   for (n1 = 0; n1 < pedsize; ++n1)
   {
      if (nghd[n1].component == comp_no)
      {
         iptr->next = (Indlist *) malloc (sizeof (Indlist));
         alloc_chk (WHERE, iptr->next);

         iptr = iptr->next;
         iptr->current = &nghd[n1];
         if (isNullNode (nghd[n1].pa))
            ++cmplist[comp_no].nfou;
      }
   }

   iptr->next = NULL;

   return;
}

/******************************************************************************/
/*                                                                            */
/* The find_comp_size subroutine is called to count the number of individuals */
/* in the component indicated by the component number passed as the first     */
/* argument to the subroutine.                                                */
/*                                                                            */
/* Before calling find_comp_size the entries in nghd's must be arranged such  */
/* that all of the individuals in each component are grouped together.        */
/*                                                                            */
/******************************************************************************/

int find_comp_size (int comp_no)
{
   int   same_comp;                     /* Initialized to TRUE.  Then set to  */
   /*                                      FALSE when the last member of the  */
   /*                                      component has been found.          */

   int   comp_size;                     /* Count of individuals in component. */

   int   i1;                            /* Current individual's subscript.    */
   int   i2;                            /* Next individual's subscript.       */

   same_comp = TRUE;
   comp_size = 0;

   for (i1 = 0; (same_comp == TRUE && i1 < pedsize); ++i1)
   {
      if (nghd[i1].component == comp_no)
      {
         ++comp_size;

         for (i2 = i1 + 1; (same_comp == TRUE && i2 < pedsize); ++i2)
         {
            if (nghd[i2].component == comp_no)
            {
               ++comp_size;

               /* See if this individual is the last one in the list.  If so, */
               /* then set same_comp to get out of both of the for loops.     */
               if ((i2 + 1) == pedsize)
                  same_comp = FALSE;
            }
            else
               same_comp = FALSE;
         }
      }
   }

   cmplist_init (comp_no, comp_size);

   return comp_size;
}

int infect (int compnt_no)
/* Infection routine for component.  It returns the size of the component.    */
{
   int   nnc, nc;
   int   i, j;

   /* Find the first member of the new component.                             */

   i = 0;
   while (i < pedsize)
   {
      if (nghd[i].component == -1)
         break;
      ++i;
   }
   if (i == pedsize)
      give_up_where (WHERE, ERN_cmp, "infect");

   nghd[i].component = compnt_no;

   /* Find the other members of the component, if any.                        */

   nnc = 1;
   nc = 0;

   while (nc < nnc)
   {
      nc = nnc;
      for (j = i + 1; j < pedsize; ++j)
      {
         if (nghd[j].component == -1)
         {
            if (neighbour (j, compnt_no) == 1)
            {
               nghd[j].component = compnt_no;
               ++nnc;
            }
         }
      }
   }

   cmplist_init (compnt_no, nnc);

   return (nnc);
}

int components (void)
{
   int   k, ncomp, nind, done;
   ncomp = 0;
   done = 0;

   while (done < pedsize)
   {
      ++ncomp;

      if (FromIndivsFile == TRUE)
         nind = find_comp_size (ncomp);
      else
         nind = infect (ncomp);

      done += nind;
      if (nind > max_cpsize)
         max_cpsize = nind;
   }

   if (DebugNghds)
   {
      for (k = 1; k <= ncomp; ++k)
         prnt_component (k);
   }

   return (ncomp);
}

/* prnt_components -- added EAT June 2008 for checking */

void prnt_component (int k)
{
   Indlist *iptr;
   int   i;

   printf ("\n Component list for component %d size %d \n", k, cmplist[k].size);
   fflush (stdout);

   iptr = cmplist[k].people;
   iptr = iptr->next;                   /* An OLD glytch!!!  EAT June 2008 */
   while (iptr)
   {
      i = iptr->current->index;
      printf (" %s", nghd[i].self);
      iptr = iptr->next;
   }
   fflush (stdout);

   return;
}

/* ------- free_cmplists:  frees the component lists (Indlists etc.) --- */

void free_cmplists (void)
{
   int   i;
   Indlist *pcurr, *pnext;

   free_1d ((void *) &pCompTab);

   if (!cmplist)
      return;

   for (i = 1; i <= n_compnts; ++i)
   {
      pnext = cmplist[i].people;
      free ((void *) pnext->current);
      cmplist[i].people = NULL;
      while (pnext)
      {
         pcurr = pnext;
         pnext = pcurr->next;
         pcurr->current = NULL;
         pcurr->next = NULL;
         free ((void *) pcurr);
      }

      if (i == 1)
         if (cmplist[i].meioses)
            free ((void *) cmplist[i].meioses);

      if (cmplist[i].ind_aff)
         free ((void *) cmplist[i].ind_aff);

      if (cmplist[i].ind_unaff)
         free ((void *) cmplist[i].ind_unaff);
   }
   free ((void *) cmplist);
   cmplist = NULL;

   return;
}
