/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*
 * proc_disp() checks the graphics display statements and sets up
 * global variables for these displays.
 */
int proc_disp (void)
{
   if (!GDispOpt)
      return TRUE;

   if (!test_gdisplay ())
      if (proc_err ())
         return FALSE;
   if (n_err)
      return proc_end ();

   mak_gdispnam ();
   mak_gdisploc ();
   prnt_gdisplay ();

   if (n_err)
      return proc_end ();
   else
      return TRUE;
}

/**
 * test_gdisplay() checks the specified display parameters.
 */
int test_gdisplay (void)
{
   if (!first_dispnam_node)
      err_typ = ERG_NDN;
   else if (first_dispnam_node->p_next)
   {
      err_typ = ERG_TMC;
      sprintf (errnumb, "%d", first_dispnam_node->component);
      err_txt = errnumb;
   }
   if (err_typ)
      return FALSE;

   if (!GDispIt)
      GDispIt = 1;
   else if (GDispIt)
      if (GDispIt > NIt)
         err_typ = ERG_TMI;

   if (err_typ)
      return FALSE;
   else
      return TRUE;
}

/**
 * mak_gdispnam () sets up the data structures specifying the individuals
 * to be displayed.
 */
void mak_gdispnam (void)
{
   int   i, c;
   char *p;
   INDNAM_PTR p_d;

   if (GDispOpt)
   {
      if (!GDispIt)                     /* default is to display every iter   */
         GDispIt = 1;

      p_d = first_dispnam_node;
      while (p_d->p_next)               /* sets no. of components with        */
         p_d = p_d->p_next;             /* individuals to display             */

      GDispNamLen = 0;                  /* no. columns needed for names       */
      NGDispCmp = p_d->component;       /* (largest component no.)            */
      NGDispNams = malloc (NGDispCmp * sizeof (int)); /* allocs for componnts */
      alloc_chk (WHERE, NGDispNams);
      GDispNam = malloc (NGDispCmp * sizeof (char **));
      alloc_chk (WHERE, GDispNam);
      GDispIx = malloc (NGDispCmp * sizeof (int *));
      alloc_chk (WHERE, GDispIx);

      p_d = first_dispnam_node;
      for (c = 0; c < NGDispCmp; ++c)   /* sets no. of indivs for ea componnt */
      {
         if (p_d->component == c + 1)
         {
            NGDispNams[c] = p_d->no_nams; /* allocs for no. individuals       */
            GDispNam[c] = malloc (NGDispNams[c] * sizeof (char *));
            alloc_chk (WHERE, GDispNam[c]);
            GDispIx[c] = malloc (NGDispNams[c] * sizeof (int));
            alloc_chk (WHERE, GDispIx[c]);

            p = p_d->p_namlist;

            for (i = 0; i < NGDispNams[c]; ++i) /* stores names               */
            {
               GDispNam[c][i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
               alloc_chk (WHERE, GDispNam[c][i]);
               strncpy (GDispNam[c][i], p, MAXNAMLEN + 1);
               if (GDispNamLen < strlen (p))
                  GDispNamLen = strlen (p);
               p += MAXNAMLEN + 1;
            }
            p_d = p_d->p_next;
         }
         else
         {
            NGDispNams[c] = 0;          /* none named for this component      */
            GDispNam[c] = NULL;
            GDispIx[c] = NULL;
         }
      }
   }
   return;
}

/**
 * mak_gdisploc() sets up the data structures specifying the loci 
 * to be displayed
 */
void mak_gdisploc (void)
{
   int   i;

   NGDispLoc = NMarks + NTraits;
   GDispLoc = malloc (NGDispLoc * sizeof (int));
   alloc_chk (WHERE, GDispLoc);
   for (i = 0; i < NGDispLoc; ++i)
   {
      if (i < Loc0)
         GDispLoc[i] = i + 1;
      else if (i == Loc0)
         GDispLoc[i] = 0;
      else
         GDispLoc[i] = i;
   }

   return;
}

/**
 * prnt_gdisplay() reports the display specified. 
 */
void prnt_gdisplay (void)
{
   int   i;

   if (GDispOpt == DEFLT_DISP)
      fprintf (stdout, "\n Segregations and founder labels");
   else if (GDispOpt == SEGR_DISP)
      fprintf (stdout, "\n Segregations");
   else if (GDispOpt == LABL_DISP)
      fprintf (stdout, "\n Founder labels");
   if (GDispOpt)
   {
      i = 0;
      while (i < NGDispCmp && !NGDispNams[i])
         ++i;
      fprintf (stdout, " are to be displayed for %d individuals\n"
               " in pedigree component %d\n\n", NGDispNams[i], i + 1);
   }
   fflush (stdout);
   return;
}
