/* map functions names:
  map                - this is a map function
     _mk             - map markers
     _tl             - map tlocs
        _mk          -   relative to marker positions
        _tst         -   test positions:
            _am      -     at marker numbers
            _ib      -     beginning positions
            _ie      -     ending positions
            _ip      -     interval proportions
            _ipa     -     all interval proportions
            _ix      -     external positions
            _jnt     -     joint positions
                _u   -       1st tloc is unlinked
            _no      -   no test positions for:
                _ip  -     interval proportions
                _xp  -     external positions
        _unlnkd      -   unlinked

   suffixes for most map function names:
        _dH          - Haldane distances
        _dK          - Kosambi distances
        _pH          - Haldane positions
        _pK          - Kosambi positions
        _rf          - recombination fractions
        _bp          - positions in base pairs

        _0           - no chromosome specified, sex-averaged map
        _0f          - no chromosome specified, female map
        _0m          - no chromosome specified, male map
        _c           - chromosome no. specified, sex-averaged map
        _cf          - chromosome no. specified, female map
        _cf          - chromosome no. specified, male map
*/

int map_mk_dH_0 (void)                  /* map marker distances               */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, AVG);
         alloc_dist (p_map, DM, MARK_LOC);
         stor_map (p_map, NCH, DM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_dH_0f (void)                 /* map marker distances (f)           */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, FEM);
         alloc_dist (p_map, DM, MARK_LOC);
         stor_map (p_map, NCH, DM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_dH_0m (void)                 /* map marker distances (m)           */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, MAL);
         alloc_dist (p_map, DM, MARK_LOC);
         stor_map (p_map, NCH, DM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_dH_c (void)                  /* map marker distances on chrom      */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, AVG);
         alloc_dist (p_map, DM, MARK_LOC);
         stor_map (p_map, CH, DM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_dH_cf (void)                 /* map marker distances on chrom (f)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, FEM);
         alloc_dist (p_map, DM, MARK_LOC);
         stor_map (p_map, CH, DM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_dH_cm (void)                 /* map marker distances on chrom (m)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, MAL);
         alloc_dist (p_map, DM, MARK_LOC);
         stor_map (p_map, CH, DM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_dK_0 (void)                  /* map marker distances (Kosambi)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, AVG);
         alloc_dist (p_map, KDM, MARK_LOC);
         stor_map (p_map, NCH, KDM, MARK_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_mk_dK_0f (void)                 /* map marker distances (Kosambi)(f)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, FEM);
         alloc_dist (p_map, KDM, MARK_LOC);
         stor_map (p_map, NCH, KDM, MARK_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_mk_dK_0m (void)                 /* map marker distances (Kosambi)(m)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, MAL);
         alloc_dist (p_map, KDM, MARK_LOC);
         stor_map (p_map, NCH, KDM, MARK_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_mk_dK_c (void)                  /* map mrkr dists on chrom (Kosambi)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, AVG);
         alloc_dist (p_map, KDM, MARK_LOC);
         stor_map (p_map, CH, KDM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_dK_cf (void)                 /* map mrkr dists on chrom (Kos) (f)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, FEM);
         alloc_dist (p_map, KDM, MARK_LOC);
         stor_map (p_map, CH, KDM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_dK_cm (void)                 /* map mrkr dists on chrom (Kos) (m)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, MAL);
         alloc_dist (p_map, KDM, MARK_LOC);
         stor_map (p_map, CH, KDM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_pH_0 (void)                  /* map marker positions               */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, PM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, AVG);
         alloc_dist (p_map, PM, MARK_LOC);
         stor_map (p_map, NCH, PM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_pH_0f (void)                 /* map marker positions (f)           */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, PM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, FEM);
         alloc_dist (p_map, PM, MARK_LOC);
         stor_map (p_map, NCH, PM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_pH_0m (void)                 /* map marker positions  (m)          */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, PM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, MAL);
         alloc_dist (p_map, PM, MARK_LOC);
         stor_map (p_map, NCH, PM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_pH_c (void)                  /* map marker positions on chrom      */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, PM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, AVG);
         alloc_dist (p_map, PM, MARK_LOC);
         stor_map (p_map, CH, PM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_pH_cf (void)                 /* map marker positions on chrom (f)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, PM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, FEM);
         alloc_dist (p_map, PM, MARK_LOC);
         stor_map (p_map, CH, PM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_pH_cm (void)                 /* map marker positions on chrom (m)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, PM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, MAL);
         alloc_dist (p_map, PM, MARK_LOC);
         stor_map (p_map, CH, PM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_pK_0 (void)                  /* map marker positions (Kosambi)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KPM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, AVG);
         alloc_dist (p_map, KPM, MARK_LOC);
         stor_map (p_map, NCH, KPM, MARK_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_mk_pK_0f (void)                 /* map marker positions (Kosambi)(f)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KPM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, FEM);
         alloc_dist (p_map, KPM, MARK_LOC);
         stor_map (p_map, NCH, KPM, MARK_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_mk_pK_0m (void)                 /* map marker positions (Kosambi)(m)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KPM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, MAL);
         alloc_dist (p_map, KPM, MARK_LOC);
         stor_map (p_map, NCH, KPM, MARK_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_mk_pK_c (void)                  /* map marker pos on chrom (Kosambi)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KPM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, AVG);
         alloc_dist (p_map, KPM, MARK_LOC);
         stor_map (p_map, CH, KPM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_pK_cf (void)                 /* map mrkr pos on chrom (Kosambi)(f) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KPM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, FEM);
         alloc_dist (p_map, KPM, MARK_LOC);
         stor_map (p_map, CH, KPM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_pK_cm (void)                 /* map mrkr pos on chrom (Kosambi)(m) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KPM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, MAL);
         alloc_dist (p_map, KPM, MARK_LOC);
         stor_map (p_map, CH, KPM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_rf_0 (void)                  /* map marker recombination fractions */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, AVG);
         alloc_dist (p_map, RM, MARK_LOC);
         stor_map (p_map, NCH, RM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_rf_0f (void)                 /* map marker recomb fracts (f)       */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, FEM);
         alloc_dist (p_map, RM, MARK_LOC);
         stor_map (p_map, NCH, RM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_rf_0m (void)                 /* map marker recomb fracts (m)       */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, MAL);
         alloc_dist (p_map, RM, MARK_LOC);
         stor_map (p_map, NCH, RM, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_rf_c (void)                  /* map marker recomb fracts on chrom  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, AVG);
         alloc_dist (p_map, RM, MARK_LOC);
         stor_map (p_map, CH, RM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_rf_cf (void)                 /* map mrk recomb fracts on chrom (f) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, FEM);
         alloc_dist (p_map, RM, MARK_LOC);
         stor_map (p_map, CH, RM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_rf_cm (void)                 /* map mrk recomb fracts on chrom (m) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, MAL);
         alloc_dist (p_map, RM, MARK_LOC);
         stor_map (p_map, CH, RM, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_mk_bp_0 (void)                  /* map marker positions base pairs    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, BP, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MARK_LOC, AVG);
         alloc_dist (p_map, BP, MARK_LOC);
         stor_map (p_map, NCH, BP, MARK_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_mk_bp_c (void)                  /* map marker positions bp's on chrom */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, BP, MARK_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, MARK_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MARK_LOC, AVG);
         alloc_dist (p_map, BP, MARK_LOC);
         stor_map (p_map, CH, BP, MARK_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_tl_mk_dH_0 (void)               /* map tloc marker distances          */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, DM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, AVG);
         alloc_dist (p_map, DM, TLOC_LOC);
         stor_map (p_map, NCH, DM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_dH_0f (void)              /* map tloc marker distances (f)      */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, DM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, FEM);
         alloc_dist (p_map, DM, TLOC_LOC);
         stor_map (p_map, NCH, DM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_dH_0m (void)              /* map tloc marker distances (m)      */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, DM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, MAL);
         alloc_dist (p_map, DM, TLOC_LOC);
         stor_map (p_map, NCH, DM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_dH_c (void)               /* map tloc marker dists on chrom     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, TLOC_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, AVG);
         alloc_dist (p_map, DM, TLOC_LOC);
         stor_map (p_map, CH, DM, TLOC_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_tl_mk_dH_cf (void)              /* map tloc marker dists on chrom (f) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, TLOC_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, FEM);
         alloc_dist (p_map, DM, TLOC_LOC);
         stor_map (p_map, CH, DM, TLOC_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_tl_mk_dH_cm (void)              /* map tloc marker dists on chrom (m) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, TLOC_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, MAL);
         alloc_dist (p_map, DM, TLOC_LOC);
         stor_map (p_map, CH, DM, TLOC_LOC); /* stores stores map             */
      }
   }
   return err_typ;
}

int map_tl_mk_dK_0 (void)               /* map tloc marker distances (Kos)    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, KDM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, AVG);
         alloc_dist (p_map, KDM, TLOC_LOC);
         stor_map (p_map, NCH, KDM, TLOC_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_tl_mk_dK_0f (void)              /* map tloc marker dists (Kosambi)(f) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, KDM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, FEM);
         alloc_dist (p_map, KDM, TLOC_LOC);
         stor_map (p_map, NCH, KDM, TLOC_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_tl_mk_dK_0m (void)              /* map tloc marker dists (Kosambi)(m) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, KDM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, MAL);
         alloc_dist (p_map, KDM, TLOC_LOC);
         stor_map (p_map, NCH, KDM, TLOC_LOC); /* stores map                  */
      }
   }
   return err_typ;
}

int map_tl_mk_dK_c (void)               /* map tloc mrkr d on chroms (Kos)    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, TLOC_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, AVG);
         alloc_dist (p_map, KDM, TLOC_LOC);
         stor_map (p_map, CH, KDM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_dK_cf (void)              /* map tloc mrkr d on chrom (Kos) (f) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, TLOC_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, FEM);
         alloc_dist (p_map, KDM, TLOC_LOC);
         stor_map (p_map, CH, KDM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_dK_cm (void)              /* map tloc mrkr d on chrom (Kos) (m) */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, TLOC_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, MAL);
         alloc_dist (p_map, KDM, TLOC_LOC);
         stor_map (p_map, CH, KDM, TLOC_LOC); /* stores stores map            */
      }
   }
   return err_typ;
}

int map_tl_mk_rf_0 (void)               /* map tloc marker recomb fracts      */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, RM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, AVG);
         alloc_dist (p_map, RM, TLOC_LOC);
         stor_map (p_map, NCH, RM, TLOC_LOC); /*stores map                    */
      }
   }
   return err_typ;
}

int map_tl_mk_rf_0f (void)              /* map tloc marker recomb fracts (f)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, RM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, FEM);
         alloc_dist (p_map, RM, TLOC_LOC);
         stor_map (p_map, NCH, RM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_rf_0m (void)              /* map tloc marker recomb fracts (m)  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (NCH, RM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (NCH, TLOC_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TLOC_LOC, MAL);
         alloc_dist (p_map, RM, TLOC_LOC);
         stor_map (p_map, NCH, RM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_rf_c (void)               /* map tloc mrkr rfs on chroms        */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, TLOC_LOC))
      {                                 /* tests for existing node           */
         if ((p_map = chk_map_node (CH, TLOC_LOC, AVG)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, AVG);
         alloc_dist (p_map, RM, TLOC_LOC);
         stor_map (p_map, CH, RM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_mk_rf_cf (void)              /* map tloc mrkr rfs on chroms (f)    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, TLOC_LOC))
      {                                 /* tests for existing node            */
         if ((p_map = chk_map_node (CH, TLOC_LOC, FEM)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, FEM);
         alloc_dist (p_map, RM, TLOC_LOC);
         stor_map (p_map, CH, RM, TLOC_LOC); /* stores map                    */
      }
   }
   return err_typ;
}

int map_tl_mk_rf_cm (void)              /* map tloc mrkr rfs on chroms (m)    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, TLOC_LOC))
      {                                 /* tests for existing node           */
         if ((p_map = chk_map_node (CH, TLOC_LOC, MAL)))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TLOC_LOC, MAL);
         alloc_dist (p_map, RM, TLOC_LOC);
         stor_map (p_map, CH, RM, TLOC_LOC); /* stores map                   */
      }
   }
   return err_typ;
}

int map_tl_tst_am_0 (void)              /* map tloc test at markers           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (!SEL_MARK)
      {
         if (chk_sim (NCH, TLOC, SS_MKLIST))
            make_sim_node (NCH, MN, SS_MKLIST);
      }

      if (chk_jmap (NCH, MN))
      {

/* If there is not already a vmap node allocated, allocate one.               */
         if (!(p_vmap = chk_vmap_node (NCH, AVG)))
            p_vmap = add_vmap_node (NCH, AVG);

/* If there was already a vmap node, check for a previously allocated array   */
/* of at marker numbers.  If there was such an array, free up its storage.    */
         else if (p_vmap->p_marker_nums)
            free (p_vmap->p_marker_nums);

/* Allocate an array for storing the at marker numbers.  Then store the at    */
/* marker numbers in the array.  The second argument to stor_vmarknums is the */
/* subscript into the parser's j_par array where the first at marker number   */
/* is located.                                                                */
         alloc_vmarknums (p_vmap);
         stor_vmarknums (p_vmap, NCH, MN);

      }
   }
   return err_typ;
}

int map_tl_tst_am_c (void)              /* map tloc test at markers on chrom  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (!SEL_MARK)
      {
         if (chk_sim (CH, TLOC, SS_MKLIST))
            make_sim_node (CH, MN, SS_MKLIST);
      }

      if (chk_jmap (CH, MN))
      {

/* If there is not already a vmap node allocated, allocate one.               */
         if (!(p_vmap = chk_vmap_node (CH, AVG)))
            p_vmap = add_vmap_node (CH, AVG);

/* If there was already a vmap node, check for a previously allocated array   */
/* of at marker numbers.  If there was such an array, free up its storage.    */
         else if (p_vmap->p_marker_nums)
            free (p_vmap->p_marker_nums);

/* Allocate an array for storing the at marker numbers.  Then store the at    */
/* marker numbers in the array.  The second argument to stor_vmarknums is the */
/* subscript into the parser's j_par array where the first at marker number   */
/* is located.                                                                */
         alloc_vmarknums (p_vmap);
         stor_vmarknums (p_vmap, CH, MN);

      }
   }
   return err_typ;
}

int map_tl_tst_ib_dH_0 (void)           /* map tloc starting distances        */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_dH_0f (void)          /* map tloc starting distances (f)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_dH_0m (void)          /* map tloc starting distances (m)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_dH_c (void)           /* map tloc starting dists on chrom   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_dH_cf (void)          /* map tloc start dists on chrom (f)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_dH_cm (void)          /* map tloc start dists on chrom (m)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_dK_0 (void)           /* map tloc starting dists (Kosambi)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fncs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_dK_0f (void)          /* map tloc start dists (Kosambi)(f)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fncs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_dK_0m (void)          /* map tloc start dists (Kosambi)(m)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fncs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_dK_c (void)           /* map tloc start d on chrom (Kos)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_dK_cf (void)          /* map tloc start d on chrom (Kos)(f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_dK_cm (void)          /* map tloc start d on chrom (Kos)(m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_rf_0 (void)           /* map tloc starting recomb fracts    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_rf_0f (void)          /* map tloc starting recomb frcts (f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_rf_0m (void)          /* map tloc starting recomb frcts (m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ib_rf_c (void)           /* map tloc start recomb frcts on chr */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_rf_cf (void)          /* map tloc starting rfs on chrom (f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ib_rf_cm (void)          /* map tloc starting rfs on chrom (m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map   */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_dH_0 (void)           /* map tloc ending distances          */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_dH_0f (void)          /* map tloc ending distances (f)      */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_dH_0m (void)          /* map tloc ending distances (m)      */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_dH_c (void)           /* map tloc ending distances on chrom */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_dH_cf (void)          /* map tloc ending dists on chrom (f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_dH_cm (void)          /* map tloc ending dists on chrom (m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_dK_0 (void)           /* map tloc ending dists (Kosambi)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fncs */
         stor_vdist (p_vmap->p_end, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_dK_0f (void)          /* map tloc ending dists (Kosambi)(f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fncs */
         stor_vdist (p_vmap->p_end, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_dK_0m (void)          /* map tloc ending dists (Kosambi)(m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fncs */
         stor_vdist (p_vmap->p_end, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_dK_c (void)           /* map tloc end dists on chrom (Kos)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_dK_cf (void)          /* map tloc end d on chrom (Kos)(f)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_dK_cm (void)          /* map tloc end d on chrom (Kos)(m)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_rf_0 (void)           /* map tloc ending recomb fracts      */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_rf_0f (void)          /* map tloc ending recomb fracts (f)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_rf_0m (void)          /* map tloc ending recomb fracts (m)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tl_tst_ie_rf_c (void)           /* map tloc ending recomb frcts on ch */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_rf_cf (void)          /* map tloc ending rfs on chrom (f)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ie_rf_cm (void)          /* map tloc ending rfs on chrom (m)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tl_tst_ip_0 (void)              /* map tloc interval proportions      */
{
   int   i;
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (NCH, IM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);

         for (i = 0; i < nk_par; ++i)   /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if ((p_vint = chk_vint (p_vmap, NCH, IM, j_par[i + 1])))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval     */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tl_tst_ip_0f (void)             /* map tloc interval proportions (f)  */
{
   int   i;
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (NCH, IM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);

         for (i = 0; i < nk_par; ++i)   /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if ((p_vint = chk_vint (p_vmap, NCH, IM, j_par[i + 1])))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval     */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tl_tst_ip_0m (void)             /* map tloc interval proportions (m)  */
{
   int   i;
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (NCH, IM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);

         for (i = 0; i < nk_par; ++i)   /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if ((p_vint = chk_vint (p_vmap, NCH, IM, j_par[i + 1])))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval     */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tl_tst_ip_c (void)              /* map tloc interval prop on chrom    */
{
   int   i;
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, IM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);

         for (i = 1; i <= nk_par; ++i)  /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if ((p_vint = chk_vint (p_vmap, CH, IM, j_par[i + 1])))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval     */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tl_tst_ip_cf (void)             /* map tloc intvl prop on chrom (f)   */
{
   int   i;
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, IM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);

         for (i = 1; i <= nk_par; ++i)  /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if ((p_vint = chk_vint (p_vmap, CH, IM, j_par[i + 1])))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval     */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tl_tst_ip_cm (void)             /* map tloc intvl prop on chrom (m)   */
{
   int   i;
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, IM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);

         for (i = 1; i <= nk_par; ++i)  /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if ((p_vint = chk_vint (p_vmap, CH, IM, j_par[i + 1])))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval     */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tl_tst_ipa_0 (void)             /* map tloc all interval proportions  */
{
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, AM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         if ((p_vint = chk_vint (p_vmap, NCH, AM, 0))) /* checks for intv "0" */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"     */
         alloc_vprop (p_vint);          /* allocs and stores list             */
         stor_vprop (p_vint);
      }
   }
   return err_typ;
}

int map_tl_tst_ipa_0f (void)            /* map tloc all interval prop (f)     */
{
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, AM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         if ((p_vint = chk_vint (p_vmap, NCH, AM, 0))) /* checks for intv "0" */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"     */
         alloc_vprop (p_vint);          /* allocs and stores list             */
         stor_vprop (p_vint);
      }
   }
   return err_typ;
}

int map_tl_tst_ipa_0m (void)            /* map tloc all interval prop (m)     */
{
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, AM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         if ((p_vint = chk_vint (p_vmap, NCH, AM, 0))) /* checks for intv "0" */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"     */
         alloc_vprop (p_vint);          /* allocs and stores list             */
         stor_vprop (p_vint);
      }
   }
   return err_typ;
}

int map_tl_tst_ipa_c (void)             /* map tloc all intervl prop on chrom */
{
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, AM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         if ((p_vint = chk_vint (p_vmap, CH, AM, 0))) /* checks for intv "0"  */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"     */
         alloc_vprop (p_vint);          /* allocs and stores list             */
         stor_vprop (p_vint);
      }
   return err_typ;
}

int map_tl_tst_ipa_cf (void)            /* map tloc all intv prop on chrm (f) */
{
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, AM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         if ((p_vint = chk_vint (p_vmap, CH, AM, 0))) /* checks for intv "0"  */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"     */
         alloc_vprop (p_vint);          /* allocs and stores list             */
         stor_vprop (p_vint);
      }
   return err_typ;
}

int map_tl_tst_ipa_cm (void)            /* map tloc all intv prop on chrm (m) */
{
   VMAP_PTR p_vmap;
   IMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, AM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         if ((p_vint = chk_vint (p_vmap, CH, AM, 0))) /* checks for intv "0"  */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"     */
         alloc_vprop (p_vint);          /* allocs and stores list             */
         stor_vprop (p_vint);
      }
   return err_typ;
}

int map_tl_tst_ix_dH_0 (void)           /* map tloc external distances        */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_dH_0f (void)          /* map tloc external distances (f)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_dH_0m (void)          /* map tloc external distances (m)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_dH_c (void)           /* map tloc external dists on chrom   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_dH_cf (void)          /* map tloc extern dists on chrom (f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_dH_cm (void)          /* map tloc extern dists on chrom (m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_dK_0 (void)           /* map tloc external dists (Kosambi)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_dK_0f (void)          /* map tloc external dists (Kos)(f)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_dK_0m (void)          /* map tloc external dists (Kos)(m)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_dK_c (void)           /* map tloc extrn dists on chrm (Kos) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_dK_cf (void)          /* map tloc ext dists on chr (Kos)(f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_dK_cm (void)          /* map tloc ext dists on chr (Kos)(m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_rf_0 (void)           /* map tloc external recomb fracts    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_rf_0f (void)          /* map tloc extern recomb fracts (f)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_rf_0m (void)          /* map tloc extern recomb fracts (m)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_vmap (NCH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (NCH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* Checks for ending map.      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tl_tst_ix_rf_c (void)           /* map tloc ext recomb fracts on chrm */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_rf_cf (void)          /* map tloc ext rec frcts on chrm (f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   return err_typ;
}

int map_tl_tst_ix_rf_cm (void)          /* map tloc ext rec frcts on chrm (m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs  */
            p_vmap = add_vmap_node (CH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map     */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* Checks for ending map.       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   return err_typ;
}

/*    **************************>>>>>>>>>>>>>>>><<<<<<<<<<*********** */

int map_tl_tst_jnt_am_0 (void)          /* map tloc test jointly at markers   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      if (chk_jmap (NCH, JMN))
      {

/* If there is not already a vmap node allocated, allocate one.               */
         if (!(p_vmap = chk_vmap_node (NCH, AVG)))
            p_vmap = add_vmap_node (NCH, AVG);

/* If there was already a vmap node, check for a previously allocated array   */
/* of at marker numbers.  If there was such an array, free up its storage.    */
         else if (p_vmap->p_marker_nums)
            free (p_vmap->p_marker_nums);

/* Set the indicator that this is a jointly at markers vmap node.             */
         p_vmap->jointly = Y;

/* Allocate an array for storing the at marker numbers.  Then store the at    */
/* marker numbers in the array.  The second argument to stor_vmarknums is the */
/* subscript into the parser's j_par array where the first at marker number   */
/* is located.                                                                */
         alloc_vmarknums (p_vmap);
         stor_vmarknums (p_vmap, NCH, JMN);

      }
   }
   return err_typ;
}

int map_tl_tst_no_ip (void)             /* map no default interval proportns  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      vmap_opt.default_nt = FALSE;
   }
   return err_typ;
}

int map_tl_tst_no_ip_c (void)           /* map no default interval proportns  */
{
   if (chk_stat ())
   {
      vmap_opt.default_nt = FALSE;
   }
   return err_typ;
}

int map_tl_tst_no_xp (void)             /* map no default external positions  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      vmap_opt.default_xt = FALSE;
   }
   return err_typ;
}

int map_tl_tst_no_xp_c (void)           /* map no default external positions  */
{
   if (chk_stat ())
   {
      vmap_opt.default_xt = FALSE;
   }
   return err_typ;
}

int map_tl_unlnkd (void)                /* map tlocs unlinked                 */
{
   UMAP_PTR p_umap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map_unlnkd ())
      {
         if ((p_umap = chk_map_unl_node ())) /* tests for existing node       */
            free_udist (p_umap);        /* (no dists in dist list)            */
         else
            p_umap = add_map_unl_node ();
         alloc_unl_dist (p_umap);
         stor_map_unl (p_umap);         /* stores unlinked tloc list          */
      }
   }
   return err_typ;
}
