/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/****************************************************************************
 *                                                                          *
 *    Penetrance routines moved from lsample_peel.c: EAT April 2002         *
 *                                                                          *
 ****************************************************************************/

#include "lsample.h"

static double sige, konst;
static double logistic_beta, logistic_mu;

void set_qtlpen (double ve)
/* sets up constants for qtlpen  */
{
   sige = ve;
   konst = 1.0 / sqrt (PPI * sige);

   return;
}

void set_logistic (void)
/* sets up static variable beta and mu (average covariate (e.g. age)
 * of affected individual) for logisticpen()  */
{
   int   i, j;

   /*M_PI is defined to be 3.14159265358979323846 in /Headers/limdefs.h */
   logistic_beta = sqrt (3 * 259.7) / M_PI;
   logistic_mu = 0;
   j = 0;

   for (i = 0; i < pedsize; i++)
   {
      if (nghd[i].d_dat != NULL)
      {
         if (nghd[i].d_dat[0] == 2)     /* affected */
         {
            logistic_mu += nghd[i].d_dat[1];
            j++;
         }
      }
   }
   logistic_mu = logistic_mu / j;

   /*
      printf("\n beta=%f,mu=%f",logistic_beta,logistic_mu);
      getchar();
    */

   return;
}

/* Change so that we are multiplying the input vector by the penetrance
 * (we just zero out the impossible terms in this case)
 * Also change so we get the number of alleles from the locus structure
 * rather than the global n_all, allowing us to use the same routine with
 * both L and M samplers - SCH June 2001 */
void recessive (double *val, const Ind * id, const struct Locus *locus,
                double lam)
{
   int   k;
   if (locus->n_all != 2)
      fprintf (stderr,
               "\n WARNING: trait locus has %d alleles\n", locus->n_all);
   if (nghd[id->index].d_dat != NULL)
   {
      k = nghd[id->index].d_dat[0];
      if (k == 4)
      {
         val[0] = 0.0;
         val[1] = 0.0;
         val[2] = 0.0;
      }
      else if (k == 3)
      {
         val[0] = 0.0;
         val[3] = 0.0;
      }
      else if (k == 2)
      {
         val[3] = 0.0;
      }
      else if (k == 1)
      {
         val[1] = 0.0;
         val[2] = 0.0;
         val[3] = 0.0;
      }
      else
      {
         fprintf (stderr,
                  "\n UNRECOGNIZED TRAIT PHENOTYPE of %s is %d", id->self, k);
         give_up_where (WHERE, ERR_qit);
      }
   }
   return;
}

/* Added by AWG */
void incomplete (double *val, const Ind * id, const struct Locus *locus,
                 double lam)
{
   int   k;
   if (locus->n_all != 2)
      fprintf (stderr,
               "\n WARNING: trait locus has %d alleles\n", locus->n_all);

   if (nghd[id->index].d_dat != NULL)
   {
      k = nghd[id->index].d_dat[0];

      /*
         0    - homozygous for the non disease allele
         1, 2 - heterozygous: carrier of the disease allele
         3    - homozygous for the disease allele
       */

      if (k == 2)                       /* Affected */
      {
         val[0] = locus->g_mu[0] * val[0];
         val[1] = locus->g_mu[1] * val[1];
         val[2] = locus->g_mu[1] * val[2];
         val[3] = locus->g_mu[2] * val[3];
      }
      else if (k == 1)                  /* Unaffected */
      {
         val[0] = (1 - locus->g_mu[0]) * val[0];
         val[1] = (1 - locus->g_mu[1]) * val[1];
         val[2] = (1 - locus->g_mu[1]) * val[2];
         val[3] = (1 - locus->g_mu[2]) * val[3];
      }
      else
      {
         fprintf (stderr,
                  "\n UNRECOGNIZED TRAIT PHENOTYPE of %s is %d", id->self, k);
         give_up_where (WHERE, ERR_qit);
      }
   }

   return;
}

void ordinal (double *val, const Ind * id, const struct Locus *locus,
              double lam)
{
   /* This penetrance function was used in the analysis of the GAW13 */
   /* data. The ordinal trait contains 5 levels where 1 corresponds  */
   /* to low values of the trait. The penetrance values are based    */
   /* on estimates from Bill's EM algorithm which fits a sinlge      */
   /* major gene.  AWG 07/31/02                                      */
   /* Note:
      0    - homozygous for the non disease allele
      1, 2 - heterozygous: carrier of the disease allele
      3    - homozygous for the disease allele
    */

   int   k;

   if (locus->n_all != 2)
      fprintf (stderr,
               "\n WARNING: trait locus has %d alleles\n", locus->n_all);
   if (nghd[id->index].d_dat != NULL)
   {
      k = nghd[id->index].d_dat[0];

      if (k == 5)
      {
         val[0] = 0.15 * val[0];
         val[1] = 0.15 * val[1];
         val[2] = 0.15 * val[2];
         val[3] = 0.95 * val[3];
      }
      else if (k == 4)
      {
         val[0] = 0.22 * val[0];
         val[1] = 0.22 * val[1];
         val[2] = 0.22 * val[2];
         val[3] = 0.05 * val[3];
      }
      else if (k == 3)
      {
         val[0] = 0.22 * val[0];
         val[1] = 0.22 * val[1];
         val[2] = 0.22 * val[2];
         val[3] = 0.0 * val[3];
      }
      else if (k == 2)
      {
         val[0] = 0.20 * val[0];
         val[1] = 0.20 * val[1];
         val[2] = 0.20 * val[2];
         val[3] = 0.0 * val[3];
      }
      else if (k == 1)
      {
         val[0] = 0.21 * val[0];
         val[1] = 0.21 * val[1];
         val[2] = 0.21 * val[2];
         val[3] = 0.0 * val[3];
      }
      else
      {
         fprintf (stderr,
                  "\n UNRECOGNIZED TRAIT PHENOTYPE of %s is %d", id->self, k);
         give_up_where (WHERE, ERR_qit);
      }
   }
   return;
}

void liability (double *val, const Ind * id, const struct Locus *locus,
                double lam)
{
   /*  Used in the analysis of the VGEOAD data. AWG */
   int   k;

   if (locus->n_all != 2)
      fprintf (stderr,
               "\n WARNING: trait locus has %d alleles\n", locus->n_all);
   if (nghd[id->index].d_dat != NULL)
   {
      k = nghd[id->index].d_dat[0];

      if (k == 2)                       /* Affected */
      {
         /* The -1 is so that the liability classes now run from 0, ...,  n-1 */
         val[0] = PenMat[nghd[id->index].d_dat[1] - 1][0] * val[0];
         val[1] = PenMat[nghd[id->index].d_dat[1] - 1][1] * val[1];
         val[2] = PenMat[nghd[id->index].d_dat[1] - 1][1] * val[2];
         val[3] = PenMat[nghd[id->index].d_dat[1] - 1][2] * val[3];
      }
      else if (k == 1)                  /* Unaffected */
      {
         val[0] = (1.0 - PenMat[nghd[id->index].d_dat[1] - 1][0]) * val[0];
         val[1] = (1.0 - PenMat[nghd[id->index].d_dat[1] - 1][1]) * val[1];
         val[2] = (1.0 - PenMat[nghd[id->index].d_dat[1] - 1][1]) * val[2];
         val[3] = (1.0 - PenMat[nghd[id->index].d_dat[1] - 1][2]) * val[3];
      }
      else
      {
         fprintf (stderr,
                  "\n UNRECOGNIZED TRAIT PHENOTYPE of %s is %d", id->self, k);
         give_up_where (WHERE, ERR_qit);
      }
   }
   return;
}

void qtlpen (double *val, const Ind * id, const struct Locus *locus, double lam)
{
   double *g_mu, z, y;
   int   i, j, k, dex;

   if (CHK_PEEL (TRACE_LEVEL_1))
      fprintf (stderr, "Inserting penetrance probabilities for %s\n", id->self);
   dex = id->index;
   if (nghd[dex].obs)
   {
/* printf ("\n %d",id->index);  */
      g_mu = locus->g_mu;
      y = nghd[dex].q_dat[0];
      k = 0;
      for (i = 0; i < locus->n_all; i++)
         for (j = 0; j <= i; j++)
         {
            z = y - lam * g_mu[k];
            z = exp (-.5 * z * z / sige) * konst;
            val[i * locus->n_all + j] *= z; /* EAT bug fixed, 4/2002 */
            if (i != j)
               val[j * locus->n_all + i] *= z; /* EAT bug fixed. 4/2002 */
            k++;                        /* EAT fixed, 6/2003  */
/* printf (" %d %d %f ", i,j,z); fflush(stdout); */
         }
   }
   return;
}

void logisticpen (double *val, const Ind * id, const struct Locus *locus,
                  double lam)
{

   int   k, a;
   double pa;

   if (locus->n_all != 2)
      fprintf (stderr,
               "\n WARNING: trait locus has %d alleles\n", locus->n_all);

   if (nghd[id->index].d_dat != NULL)
   {
      k = nghd[id->index].d_dat[0];
      a = nghd[id->index].d_dat[1];

      /*
         0    - homozygous for the non disease allele
         1, 2 - heterozygous: carrier of the disease allele
         3    - homozygous for the disease allele
       */

      if (a <= 0)
         return;                        /* covariate (age} is not observed */

      if (k == 2)                       /* Affected */
      {
         pa = 1.0 / (1 + exp (-(a - 90.0) / logistic_beta))
            - 1.0 / (1 + exp (-(a - 90.0 - 2) / logistic_beta));
         val[0] = pa * val[0];

         pa = 1.0 / (1 + exp (-(a - logistic_mu) / logistic_beta))
            - 1.0 / (1 + exp (-(a - logistic_mu - 2) / logistic_beta));
         val[1] = pa * val[1];
         val[2] = pa * val[2];
         val[3] = pa * val[3];
      }
      else if (k == 1)                  /* Unaffected */
      {
         pa = 1.0 - 1.0 / (1 + exp (-(a - 90.0) / logistic_beta));
         val[0] = pa * val[0];

         pa = 1.0 - 1.0 / (1 + exp (-(a - logistic_mu) / logistic_beta));
         val[1] = pa * val[1];
         val[2] = pa * val[2];
         val[3] = pa * val[3];
      }
      else
      {
         fprintf (stderr,
                  "\n UNRECOGNIZED TRAIT PHENOTYPE of %s is %d", id->self, k);
         give_up_where (WHERE, ERR_qit);
      }

   }
   return;
}
