/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/***********************************************************************
 *           Called by MORGAN functions or programs.
 *
 *           give_warn()         Print warning messages.
 *           give_warn_where()
 *           give_err()          Print error messages.
 *           give_err_where()
 *           give_up()           These last 2 functions
 *           give_up_where()     also terminate processing.
 ***********************************************************************
 *  Arguments are:
 *
 *    give_warn(), give_err(), give_up():
 *        The subscript for the error message in Errrs.
 *        Zero, one or more printf arguments.
 *
 *    give_warn_where(), give_err_where(), give_up_where():
 *        The name of the file (__FILE__).
 *        The line number (__LINE__).
 *        The subscript for the error message in Errrs.
 *        Zero, one or more printf arguments.
 **********************************************************************/

#include "stuff.h"

static void v_prnt_err (int, char);
static void prnt_where (char *, int);
static void prnt_qit (void);

static va_list pargs;

void give_warn (int err_no, ...)
{
   fflush (stdout);
   va_start (pargs, err_no);
   v_prnt_err (err_no, 'W');
   va_end (pargs);

   if (++nWarns > MAX_WARNS)
      give_up (ERS_mxw);

   return;
}

void give_warn_where (char *src_fil, int lin_no, int err_no, ...)
{
   fflush (stdout);
   prnt_where (src_fil, lin_no);

   va_start (pargs, err_no);
   v_prnt_err (err_no, 'W');
   va_end (pargs);

   if (++nWarns > MAX_WARNS)
      give_up (ERS_mxw);

   return;
}

void give_err (int err_no, ...)
{
   fflush (stdout);
   va_start (pargs, err_no);
   v_prnt_err (err_no, 'E');
   va_end (pargs);

   return;
}

void give_err_where (char *src_fil, int lin_no, int err_no, ...)
{
   fflush (stdout);
   prnt_where (src_fil, lin_no);

   va_start (pargs, err_no);
   v_prnt_err (err_no, 'E');
   va_end (pargs);

   return;
}

void give_up (int err_no, ...)
{
   fflush (stdout);
   if (err_no != ERR_qit)
   {
      va_start (pargs, err_no);
      v_prnt_err (err_no, 'F');
      va_end (pargs);
   }

   prnt_qit ();
   exit (EXIT_FAILURE);
}

void give_up_where (char *src_fil, int lin_no, int err_no, ...)
{
   fflush (stdout);
   prnt_where (src_fil, lin_no);

   va_start (pargs, err_no);
   if (err_no != ERR_qit)
      v_prnt_err (err_no, 'F');
   va_end (pargs);

   prnt_qit ();
   exit (EXIT_FAILURE);
}

void v_prnt_err (int e_no, char level)
{
   vfprintf (stderr, Errrs[e_no], pargs);
   fprintf (stderr, "  (%c)\n\n", level);
   fflush (stderr);

   return;
}

void prnt_where (char *src, int lin)
{
   fprintf (stderr, " File %s, line %d:\n", src, lin);

   return;
}

void prnt_qit (void)
{
   fprintf (stderr, "%s", Errrs[ERR_qit]);
   fflush (stderr);

   return;
}
